package rs.soneco.icmynet.web.client;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import com.google.gwt.user.client.ui.IsWidget;

import rs.soneco.icmynet.dashboard.client.DashboardModule;
import rs.soneco.icmynet.events.client.EventsModule;
import rs.soneco.icmynet.flow.client.FlowModule;
import rs.soneco.icmynet.flow.client.settings.FlowSettings;
import rs.soneco.icmynet.flow.client.ui.commands.CheckArchiveAndDatabaseStatusCommand;
import rs.soneco.icmynet.flow.client.ui.commands.CheckArchiveAndDatabaseStatusResponse;
import rs.soneco.icmynet.flow.client.ui.commands.CheckDatabasePerformanceCommand;
import rs.soneco.icmynet.flow.client.ui.commands.CheckDatabasePerformanceResponse;
import rs.soneco.icmynet.mib.client.MibModule;
import rs.soneco.icmynet.mib.client.widgets.MibSettingsWidget;
import rs.soneco.icmynet.model.login.module.ModuleIdentificator;
import rs.soneco.icmynet.multilog.client.MultilogModule;
import rs.soneco.icmynet.multilog.client.MultilogSettings;
import rs.soneco.icmynet.nms.client.NmsModule;
import rs.soneco.icmynet.nms.client.settings.NmsSettings;
import rs.soneco.icmynet.view.framework.client.Framework;
import rs.soneco.icmynet.view.framework.client.command.SystemPropertiesPerUserCommand;
import rs.soneco.icmynet.view.framework.client.command.SystemPropertiesPerUserResponse;
import rs.soneco.icmynet.view.framework.client.message.MessageEvent;
import rs.soneco.icmynet.view.framework.client.message.MessageType;
import rs.soneco.icmynet.view.framework.client.module.Module;
import rs.soneco.icmynet.view.framework.client.service.ServiceResponse;
import rs.soneco.icmynet.view.framework.client.service.aa.SecureSession;
import rs.soneco.icmynet.view.framework.client.state.ItemSelectionEvent;
import rs.soneco.icmynet.view.framework.client.state.TimeSelectionEvent;
import rs.soneco.icmynet.view.framework.client.state.Timeframe;
import rs.soneco.icmynet.view.framework.client.state.Timewindow;
import rs.soneco.icmynet.view.framework.client.state.WizardInitEvent;
import rs.soneco.icmynet.view.framework.client.ui.wizard.command.LoadWizardStepsCommand;
import rs.soneco.icmynet.view.framework.client.ui.wizard.command.LoadWizardStepsResponse;
import rs.soneco.icmynet.web.client.logresource.MultiLogCss;
import rs.soneco.icmynet.web.client.mibresource.MibCss;

public class NetVizura extends Framework {

    private static final long DAY = 24L * 60L * 60L *1000L;
    
    private MibModule mib;
    private MibSettingsWidget mibSettings;
    private MultilogModule multilog;
    private MultilogSettings multilogSettings;
    private FlowModule flow;
    private FlowSettings flowSettings;
    private NmsSettings nmsSettings;
    private NmsModule nms;
    private EventsModule events;
	private DashboardModule dashboard;
    
    @Override
    protected Module provideModule(String tag) {
        if (tag.equals(MibModule.TAG)) {
            if (mib == null) {
                mib = new MibModule();
            }
            return mib;
        } else if (tag.equals(MultilogModule.TAG)) {
            if (multilog == null) {
                multilog = new MultilogModule();
            }
            return multilog;
        } else if (tag.equals(FlowModule.TAG)) {
            if (flow == null) {
                flow = new FlowModule();
            }
            return flow;
        } else if (tag.equals(NmsModule.TAG)) {
            if (nms == null) {
                nms = new NmsModule();
            }
            return nms;
        } else if (tag.equals(EventsModule.TAG)) {
            if (events == null) {
                events = new EventsModule();
            }
            return events;
        } else if (tag.equals(DashboardModule.TAG)) {
            if (dashboard == null) {
            	dashboard = new DashboardModule();
            }
            return dashboard;
        } else {
            return null;
        }
    }
    
    @Override
    protected Set<String> getAvailableModules() {
        HashSet<String> set = new HashSet<String>();
        set.add(MibModule.TAG);
        set.add(MultilogModule.TAG);
        set.add(FlowModule.TAG);
        set.add(NmsModule.TAG);
        set.add(EventsModule.TAG);
        set.add(DashboardModule.TAG);
        return set;
    }
    
    @Override
    protected IsWidget provideSettingsPanel(String tag) {
        if (tag.equals(MibModule.TAG)) {
            if (mibSettings == null) {
                mibSettings = new MibSettingsWidget();
            }
            return mibSettings;
        } else if (tag.equals(MultilogModule.TAG)) {
            if (multilogSettings == null) {
                multilogSettings = new MultilogSettings();
            }
            return multilogSettings;
        } else if (tag.equals(FlowModule.TAG)) {
            if (flowSettings == null) {
                flowSettings = new FlowSettings();
            }
            return flowSettings;
        } else if (tag.equals(NmsModule.TAG)) {
            if (nmsSettings == null) {
                nmsSettings = new NmsSettings();
            }
            return nmsSettings;
        } else {
            return null;
        }
    }
    
    @Override
    protected void initAppState() {
        MibCss.css.mib().ensureInjected();
        MultiLogCss.css.multilog().ensureInjected();
        
        // ucitavanje initial propertiesa..., dodati u request listu po potrebi
        SecureSession sessionMisc = Framework.getSession(ModuleIdentificator.GENERAL);
        if(sessionMisc == null) {
            Framework.fireEvent(new MessageEvent(MessageType.INFO, "You have no privileges set. You should contact your administrator."));
            return;
        }
        SystemPropertiesPerUserCommand cmd = new SystemPropertiesPerUserCommand(ModuleIdentificator.GENERAL, sessionMisc, false);
        cmd.getRequestProperties().add("TIMEWINDOW");
        cmd.getRequestProperties().add("DATE_FORMAT");
        Framework.execute(cmd, new ServiceResponse<SystemPropertiesPerUserResponse>() {
            @Override
            public void onResponse(SystemPropertiesPerUserResponse response) {              
                String timeWindow = response.getPropertyValues().get("TIMEWINDOW");
                int tw = Timewindow.LAST_DAY.ordinal();
				if (timeWindow != null) {
					tw = Integer.parseInt(timeWindow);
				}
				
				String timeWindowFormat = response.getPropertyValues().get("DATE_FORMAT");
				if (timeWindowFormat != null) {
					Framework.fireEvent(new ItemSelectionEvent("DATE_FORMAT", timeWindowFormat));
				}
				
                Framework.fireEvent(new TimeSelectionEvent(new Timeframe(Timewindow.values()[tw])));
                Framework.fireEvent(new ItemSelectionEvent("TIMEWINDOW", tw));
                
                // ucitani svi propertisi.
                appStateInitialized();
            }           
        });
        
        Framework.execute(new LoadWizardStepsCommand(sessionMisc), new ServiceResponse<LoadWizardStepsResponse>() {
			@Override
			public void onResponse(LoadWizardStepsResponse response) {
				if(response.isSuccess()) {
					Framework.fireEvent(new WizardInitEvent(response.isWizardMuted(), response.isWizardPanelVisible(), response.getSteps()));
				}
			}
		});
        
        final SecureSession session = Framework.getSession(FlowModule.TAG);
        if (session != null) {
            // check Flow database performance
            CheckDatabasePerformanceCommand dbCommand = new CheckDatabasePerformanceCommand(session);
            dbCommand.setCheckEnd(new Date());
            dbCommand.setCheckStart(new Date(System.currentTimeMillis()-DAY));
            Framework.execute(dbCommand, new ServiceResponse<CheckDatabasePerformanceResponse>() {
                @Override
                public void onResponse(CheckDatabasePerformanceResponse result) {
                	if (result.getMessage() != null) {
                        Framework.fireEvent(new MessageEvent(MessageType.WARNING, result.getMessage()));
                    }
                }
            });
            // check Flow archive status
            CheckArchiveAndDatabaseStatusCommand archiveCommand = new CheckArchiveAndDatabaseStatusCommand(session);
            Framework.execute(archiveCommand, new ServiceResponse<CheckArchiveAndDatabaseStatusResponse>() {
                @Override
                public void onResponse(CheckArchiveAndDatabaseStatusResponse result) {
                	if (result.getArchiveMessages() != null) {
                		HashMap<String, List<String>> details = result.getArchiveDetails();
                		if (details == null) {
                			details = new HashMap<String, List<String>>(); 
                		}
                		for (Entry<MessageType, List<String>> entry : result.getArchiveMessages().entrySet()) {
							for (String message : entry.getValue()) {
								Framework.fireEvent(new MessageEvent(entry.getKey(), message, details.get(message)));
							}
						}
                	}
                	if (result.getDatabaseMessages() != null) {
                		HashMap<String, List<String>> details = result.getDatabaseDetails();
                		if (details == null) {
                			details = new HashMap<String, List<String>>(); 
                		}
                		for (Entry<MessageType, List<String>> entry : result.getDatabaseMessages().entrySet()) {
							for (String message : entry.getValue()) {
								Framework.fireEvent(new MessageEvent(entry.getKey(), message, details.get(message)));
							}
						}
                	}
                }
            });
        }
        
    }

    @Override
    protected void destroyAppState() {
        //clear modules
        if(mib != null) {
            mib.setInactive();
            mib.removeFromParent();
            mib = null;
        }
        if(flow != null) {
            flow.stopAllTimers();
            flow.setInactive();
            flow.removeFromParent();
            flow = null;
        }
        if(multilog != null){
            multilog.setInactive();
            multilog.removeFromParent();
            multilog = null;
        }
        if(nms != null) {
            nms.setInactive();
            nms.removeFromParent();
            nms = null;
        }
        if (events != null) {
            events.setInactive();
            events.removeFromParent();
            events = null;
        }
        if(dashboard != null) {
        	dashboard.setInactive();
        	dashboard.removeFromParent();
        	dashboard = null;
        }
        //clear settings
        if(mibSettings != null) {
            mibSettings.removeFromParent();
            mibSettings = null;
        }
        if(multilogSettings != null){
            multilogSettings.removeFromParent();
            multilogSettings = null;
        }
        if (flowSettings != null) {
            flowSettings.removeFromParent();
            flowSettings = null;
        }
        if (nmsSettings != null) {
            nmsSettings.removeFromParent();
            nmsSettings = null;
        }
    }
}
