CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_as(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_as_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_as_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, "as", SUM(byte_src) AS byte_src, SUM(byte_dst) AS byte_dst, SUM(pack_src) AS pack_src, SUM(pack_dst) AS pack_dst, SUM(flow_src) AS flow_src, SUM(flow_dst) AS flow_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, "as"' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, "as", byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t2, srcdata.exporter, srcdata.as, srcdata.byte_src, srcdata.byte_dst, srcdata.pack_src, srcdata.pack_dst, srcdata.flow_src, srcdata.flow_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_as(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_conversation(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_conversation_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_conversation_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, source, destination, service, SUM(byte) AS byte, SUM(pack) AS pack, SUM(flow) AS flow FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, source, destination, service' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, source, destination, service, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6, $7, $8)' USING t2, srcdata.exporter, srcdata.source, srcdata.destination, srcdata.service, srcdata.byte, srcdata.pack, srcdata.flow;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_conversation(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;  


CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_host(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_host_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_host_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, host, SUM(byte_src) AS byte_src, SUM(byte_dst) AS byte_dst, SUM(pack_src) AS pack_src, SUM(pack_dst) AS pack_dst, SUM(flow_src) AS flow_src, SUM(flow_dst) AS flow_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, host' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, host, byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t2, srcdata.exporter, srcdata.host, srcdata.byte_src, srcdata.byte_dst, srcdata.pack_src, srcdata.pack_dst, srcdata.flow_src, srcdata.flow_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_host(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_protocol(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_protocol_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_protocol_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, protocol, SUM(byte) AS byte, SUM(pack) AS pack, SUM(flow) AS flow FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, protocol' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, protocol, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6)' USING t2, srcdata.exporter, srcdata.protocol, srcdata.byte, srcdata.pack, srcdata.flow;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_protocol(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_qos(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_qos_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_qos_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, tos, SUM(byte) AS byte, SUM(pack) AS pack, SUM(flow) AS flow FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, tos' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, tos, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6)' USING t2, srcdata.exporter, srcdata.tos, srcdata.byte, srcdata.pack, srcdata.flow;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_qos(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_service(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_service_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_service_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, port, SUM(byte_src) AS byte_src, SUM(byte_dst) AS byte_dst, SUM(pack_src) AS pack_src, SUM(pack_dst) AS pack_dst, SUM(flow_src) AS flow_src, SUM(flow_dst) AS flow_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, port' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, port, byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t2, srcdata.exporter, srcdata.port, srcdata.byte_src, srcdata.byte_dst, srcdata.pack_src, srcdata.pack_dst, srcdata.flow_src, srcdata.flow_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_service(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_total(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_total_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_total_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, SUM(byte) AS byte, SUM(pack) AS pack, SUM(flow) AS flow FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, byte, pack, flow) VALUES($1, $2, $3, $4, $5)' USING t2, srcdata.exporter, srcdata.byte, srcdata.pack, srcdata.flow;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_total(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;



CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_as(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_as_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_as_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, "as", SUM(byte_in_src) AS byte_in_src, SUM(byte_in_dst) AS byte_in_dst, SUM(byte_out_src) AS byte_out_src, SUM(byte_out_dst) AS byte_out_dst, SUM(pack_in_src) AS pack_in_src, SUM(pack_in_dst) AS pack_in_dst, SUM(pack_out_src) AS pack_out_src, SUM(pack_out_dst) AS pack_out_dst, SUM(flow_in_src) AS flow_in_src, SUM(flow_in_dst) AS flow_in_dst, SUM(flow_out_src) AS flow_out_src, SUM(flow_out_dst) AS flow_out_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface, "as"' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, "as", byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t2, srcdata.exporter, srcdata.interface, srcdata.as, srcdata.byte_in_src, srcdata.byte_in_dst, srcdata.byte_out_src, srcdata.byte_out_dst, srcdata.pack_in_src, srcdata.pack_in_dst, srcdata.pack_out_src, srcdata.pack_out_dst, srcdata.flow_in_src, srcdata.flow_in_dst, srcdata.flow_out_src, srcdata.flow_out_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_as(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;

CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_conversation(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_conversation_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_conversation_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, source, destination, service, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface, source, destination, service' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12)' USING t2, srcdata.exporter, srcdata.interface, srcdata.source, srcdata.destination, srcdata.service, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_conversation(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;  


CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_host(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_host_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_host_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, host, SUM(byte_in_src) AS byte_in_src, SUM(byte_in_dst) AS byte_in_dst, SUM(byte_out_src) AS byte_out_src, SUM(byte_out_dst) AS byte_out_dst, SUM(pack_in_src) AS pack_in_src, SUM(pack_in_dst) AS pack_in_dst, SUM(pack_out_src) AS pack_out_src, SUM(pack_out_dst) AS pack_out_dst, SUM(flow_in_src) AS flow_in_src, SUM(flow_in_dst) AS flow_in_dst, SUM(flow_out_src) AS flow_out_src, SUM(flow_out_dst) AS flow_out_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface, host' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, host, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t2, srcdata.exporter, srcdata.interface, srcdata.host, srcdata.byte_in_src, srcdata.byte_in_dst, srcdata.byte_out_src, srcdata.byte_out_dst, srcdata.pack_in_src, srcdata.pack_in_dst, srcdata.pack_out_src, srcdata.pack_out_dst, srcdata.flow_in_src, srcdata.flow_in_dst, srcdata.flow_out_src, srcdata.flow_out_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_host(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;
  

CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_protocol(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_protocol_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_protocol_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, protocol, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface, protocol' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, protocol, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t2, srcdata.exporter, srcdata.interface, srcdata.protocol, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_protocol(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_qos(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_qos_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_qos_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, tos, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface, tos' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, tos, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t2, srcdata.exporter, srcdata.interface, srcdata.tos, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_qos(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_service(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_service_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_service_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, port, SUM(byte_in_src) AS byte_in_src, SUM(byte_in_dst) AS byte_in_dst, SUM(byte_out_src) AS byte_out_src, SUM(byte_out_dst) AS byte_out_dst, SUM(pack_in_src) AS pack_in_src, SUM(pack_in_dst) AS pack_in_dst, SUM(pack_out_src) AS pack_out_src, SUM(pack_out_dst) AS pack_out_dst, SUM(flow_in_src) AS flow_in_src, SUM(flow_in_dst) AS flow_in_dst, SUM(flow_out_src) AS flow_out_src, SUM(flow_out_dst) AS flow_out_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface, port' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, port, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t2, srcdata.exporter, srcdata.interface, srcdata.port, srcdata.byte_in_src, srcdata.byte_in_dst, srcdata.byte_out_src, srcdata.byte_out_dst, srcdata.pack_in_src, srcdata.pack_in_dst, srcdata.pack_out_src, srcdata.pack_out_dst, srcdata.flow_in_src, srcdata.flow_in_dst, srcdata.flow_out_src, srcdata.flow_out_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_service(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_total(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_total_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_total_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t2, srcdata.exporter, srcdata.interface, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_total(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;  


  
 
CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_as(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_as_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_as_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, "as", SUM(byte_in_src) AS byte_in_src, SUM(byte_in_dst) AS byte_in_dst, SUM(byte_out_src) AS byte_out_src, SUM(byte_out_dst) AS byte_out_dst, SUM(pack_in_src) AS pack_in_src, SUM(pack_in_dst) AS pack_in_dst, SUM(pack_out_src) AS pack_out_src, SUM(pack_out_dst) AS pack_out_dst, SUM(flow_in_src) AS flow_in_src, SUM(flow_in_dst) AS flow_in_dst, SUM(flow_out_src) AS flow_out_src, SUM(flow_out_dst) AS flow_out_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set, "as"' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, "as", byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.as, srcdata.byte_in_src, srcdata.byte_in_dst, srcdata.byte_out_src, srcdata.byte_out_dst, srcdata.pack_in_src, srcdata.pack_in_dst, srcdata.pack_out_src, srcdata.pack_out_dst, srcdata.flow_in_src, srcdata.flow_in_dst, srcdata.flow_out_src, srcdata.flow_out_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_as(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_conversation(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_conversation_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_conversation_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, source, destination, service, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set, source, destination, service' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.source, srcdata.destination, srcdata.service, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_conversation(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;  


CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_host(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_host_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_host_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, host, SUM(byte_in_src) AS byte_in_src, SUM(byte_in_dst) AS byte_in_dst, SUM(byte_out_src) AS byte_out_src, SUM(byte_out_dst) AS byte_out_dst, SUM(pack_in_src) AS pack_in_src, SUM(pack_in_dst) AS pack_in_dst, SUM(pack_out_src) AS pack_out_src, SUM(pack_out_dst) AS pack_out_dst, SUM(flow_in_src) AS flow_in_src, SUM(flow_in_dst) AS flow_in_dst, SUM(flow_out_src) AS flow_out_src, SUM(flow_out_dst) AS flow_out_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set, host' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, host, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.host, srcdata.byte_in_src, srcdata.byte_in_dst, srcdata.byte_out_src, srcdata.byte_out_dst, srcdata.pack_in_src, srcdata.pack_in_dst, srcdata.pack_out_src, srcdata.pack_out_dst, srcdata.flow_in_src, srcdata.flow_in_dst, srcdata.flow_out_src, srcdata.flow_out_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_host(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_protocol(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_protocol_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_protocol_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, protocol, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set, protocol' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, protocol, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.protocol, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_protocol(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_qos(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_qos_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_qos_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, tos, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set, tos' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, tos, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.tos, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_qos(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_service(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_service_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_service_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, port, SUM(byte_in_src) AS byte_in_src, SUM(byte_in_dst) AS byte_in_dst, SUM(byte_out_src) AS byte_out_src, SUM(byte_out_dst) AS byte_out_dst, SUM(pack_in_src) AS pack_in_src, SUM(pack_in_dst) AS pack_in_dst, SUM(pack_out_src) AS pack_out_src, SUM(pack_out_dst) AS pack_out_dst, SUM(flow_in_src) AS flow_in_src, SUM(flow_in_dst) AS flow_in_dst, SUM(flow_out_src) AS flow_out_src, SUM(flow_out_dst) AS flow_out_dst FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set, port' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, port, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.port, srcdata.byte_in_src, srcdata.byte_in_dst, srcdata.byte_out_src, srcdata.byte_out_dst, srcdata.pack_in_src, srcdata.pack_in_dst, srcdata.pack_out_src, srcdata.pack_out_dst, srcdata.flow_in_src, srcdata.flow_in_dst, srcdata.flow_out_src, srcdata.flow_out_dst;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_service(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_total(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_total_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_total_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_total(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


  
  
CREATE OR REPLACE FUNCTION netflow.aggregate(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone) RETURNS void AS
$BODY$
DECLARE
BEGIN
  PERFORM netflow.agg_exporter_device_total(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_device_as(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_device_conversation(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_device_host(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_device_protocol(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_device_qos(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_device_service(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);

  PERFORM netflow.agg_exporter_interface_total(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_interface_as(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_interface_conversation(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_interface_host(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_interface_protocol(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_interface_qos(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_exporter_interface_service(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);

  PERFORM netflow.agg_pattern_subnet_total(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_pattern_subnet_as(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_pattern_subnet_conversation(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_pattern_subnet_host(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_pattern_subnet_protocol(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_pattern_subnet_qos(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.agg_pattern_subnet_service(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.aggregate(text, text, text, text, timestamp without time zone, timestamp without time zone) OWNER TO postgres;


INSERT INTO framework.db_version(patch_number, patch_date) VALUES(8, current_timestamp);