CREATE OR REPLACE FUNCTION multilog.syslog_message_count(IN t1 bigint, IN t2 bigint, IN exporters text, IN severities text, IN facilities text, IN message text)
  RETURNS TABLE(t timestamp without time zone, exp inet, sev integer, cnt bigint) AS
$BODY$
DECLARE
  exporter_filter inet[];
  severity_filter int[];
  facility_filter int[];
  time1 timestamp;
  time2 timestamp;
  query text;
BEGIN
  IF (exporters IS NOT NULL) THEN
    exporter_filter := string_to_array(exporters,',');
  END IF;
  IF (severities IS NOT NULL) THEN
    severity_filter := string_to_array(severities,',');
  END IF;
  IF (facilities IS NOT NULL) THEN
    facility_filter := string_to_array(facilities,',');
  END IF;
  SELECT multilog.timestamp_from_millis(t1) INTO time1;
  SELECT multilog.timestamp_from_millis(t2) INTO time2;
  query := 'SELECT ''' || time2 || '''::timestamp, exporter, severity, count(1) AS cnt FROM multilog.syslog_message WHERE "time" >= $1 AND "time" < $2';
  IF (exporter_filter IS NOT NULL) THEN
	query := query || ' AND exporter = ANY($3)';
  END IF;
  IF (severity_filter IS NOT NULL) THEN 
    query := query || ' AND severity = ANY($4)';
  END IF;
  IF (facility_filter IS NOT NULL) THEN
    query := query || ' AND facility = ANY($5)';
  END IF;
  IF (message IS NOT NULL) THEN
    query := query || ' AND LOWER(message) LIKE $6';
  END IF;
  query := query || ' GROUP BY exporter, severity';
  RETURN QUERY EXECUTE query USING time1, time2, exporter_filter, severity_filter, facility_filter, message || '%';
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION multilog.syslog_message_count(bigint, bigint, text, text, text, text) OWNER TO postgres;

INSERT INTO framework.db_version(patch_number, patch_date) VALUES(10, current_timestamp);