DO $$DECLARE 
  r_multidevices record;
  r_devices record;
  r_subdevices record;
  device_id integer;
BEGIN
  FOR r_multidevices IN select ip_address, count(*) as broj_redova from device.device GROUP BY ip_address having count(*) > 1
  LOOP
          -- izaberi prvi device koji je network exporter
    select id_device into device_id from (
      SELECT
    	d.id_device,
    	row_number() OVER () broj_reda
      FROM device.device d
      INNER JOIN typology.device_device_type ddt ON ddt.id_device = d.id_device
      WHERE ip_address = r_multidevices.ip_address
      AND ddt.id_device_type = 2
      order by d.id_device
    ) pom
    where pom.broj_reda = 1;
    
    -- ako ne postoji takav, izaberi prvi device
    IF NOT FOUND THEN
      select id_device into device_id from (
        SELECT
      	id_device,
      	row_number() OVER() broj_reda
        FROM device.device
        WHERE ip_address = r_multidevices.ip_address
        ORDER BY id_device
      ) pom
      where pom.broj_reda = 1;
    END IF;
		
    if (device_id is not null) then
      -- prebaci subdevice na odabrani device
      update device.subdevice sd
        set id_device = device_id
      from device.device d
      where d.id_device = sd.id_device 
      and d.ip_address = r_multidevices.ip_address
      and d.id_device <> device_id
      ;
      
      -- prebaci resource na odabrani device
      update device.resource rd
        set id_device = device_id
      from device.device d
      where d.id_device = rd.id_device 
      and d.ip_address = r_multidevices.ip_address
      and d.id_device <> device_id
      ;
      
      -- prebaci samo tipove koji vec ne postoje na izabranom deviceu
      update typology.device_device_type ddt
        set id_device = device_id
      from device.device d
      where d.id_device = ddt.id_device 
      and d.ip_address = r_multidevices.ip_address
      and d.id_device <> device_id
      and not exists (select * from typology.device_device_type ddt2 where ddt2.id_device = device_id and ddt2.id_device_type = ddt.id_device_type)
      ;
      
      -- obrisi ostale tipove
      delete from typology.device_device_type ddt
      using device.device d
      where ddt.id_device = d.id_device
      and d.id_device <> device_id
      and d.ip_address = r_multidevices.ip_address
      ;
      
      -- prebaci samo snmp polise koje vec ne postoje na izabranom deviceu
      update policy.device_policy_snmp dps
        set id_device = device_id
      from device.device d
      where d.id_device = dps.id_device 
      and d.ip_address = r_multidevices.ip_address
      and d.id_device <> device_id
      and not exists (select * from policy.device_policy_snmp dps2 where dps2.id_device = device_id and dps2.id_policy_snmp = dps.id_policy_snmp)
      ;
      
      -- obrisi ostale snmp polise
      delete from policy.device_policy_snmp dps
      using device.device d
      where dps.id_device = d.id_device
      and d.id_device <> device_id
      and d.ip_address = r_multidevices.ip_address
      ;
      
      -- prebaci samo console polise koje vec ne postoje na izabranom deviceu
      update policy.device_policy_console dpc
        set id_device = device_id
      from device.device d
      where d.id_device = dpc.id_device 
      and d.ip_address = r_multidevices.ip_address
      and d.id_device <> device_id
      and not exists (select * from policy.device_policy_console dps2 where dps2.id_device = device_id and dps2.id_policy_console = dpc.id_policy_console)
      ;
      
      -- obrisi ostale console polise
      delete from policy.device_policy_console dpc
      using device.device d
      where dpc.id_device = d.id_device
      and d.id_device <> device_id
      and d.ip_address = r_multidevices.ip_address
      ;
      
      -- obrisi sve ostale deviceve sa datom ip adresom
      delete from device.device where id_device <> device_id and ip_address = r_multidevices.ip_address;
    end if;
  END LOOP;
END$$;


ALTER TABLE device.device ADD CONSTRAINT uq_device_ip_address UNIQUE (ip_address);

INSERT INTO framework.db_version(patch_number, patch_date) VALUES(30, current_timestamp);