-- DODAVANJE KOLONE ZA PROTOKOL U KONVERZACIJE
ALTER TABLE netflow.exporter_device_conversation_g1 ADD COLUMN protocol integer;
ALTER TABLE netflow.exporter_device_conversation_g2 ADD COLUMN protocol integer;
ALTER TABLE netflow.exporter_device_conversation_g3 ADD COLUMN protocol integer;

UPDATE netflow.exporter_device_conversation_g1 SET protocol = 0;
UPDATE netflow.exporter_device_conversation_g2 SET protocol = 0;
UPDATE netflow.exporter_device_conversation_g3 SET protocol = 0;

ALTER TABLE netflow.exporter_device_conversation_g1 ALTER COLUMN protocol SET NOT NULL;
ALTER TABLE netflow.exporter_device_conversation_g2 ALTER COLUMN protocol SET NOT NULL;
ALTER TABLE netflow.exporter_device_conversation_g3 ALTER COLUMN protocol SET NOT NULL;

ALTER TABLE netflow.exporter_interface_conversation_g1 ADD COLUMN protocol integer;
ALTER TABLE netflow.exporter_interface_conversation_g2 ADD COLUMN protocol integer;
ALTER TABLE netflow.exporter_interface_conversation_g3 ADD COLUMN protocol integer;

UPDATE netflow.exporter_interface_conversation_g1 SET protocol = 0;
UPDATE netflow.exporter_interface_conversation_g2 SET protocol = 0;
UPDATE netflow.exporter_interface_conversation_g3 SET protocol = 0;

ALTER TABLE netflow.exporter_interface_conversation_g1 ALTER COLUMN protocol SET NOT NULL;
ALTER TABLE netflow.exporter_interface_conversation_g2 ALTER COLUMN protocol SET NOT NULL;
ALTER TABLE netflow.exporter_interface_conversation_g3 ALTER COLUMN protocol SET NOT NULL;

ALTER TABLE netflow.pattern_subnet_conversation_g1 ADD COLUMN protocol integer;
ALTER TABLE netflow.pattern_subnet_conversation_g2 ADD COLUMN protocol integer;
ALTER TABLE netflow.pattern_subnet_conversation_g3 ADD COLUMN protocol integer;

UPDATE netflow.pattern_subnet_conversation_g1 SET protocol = 0;
UPDATE netflow.pattern_subnet_conversation_g2 SET protocol = 0;
UPDATE netflow.pattern_subnet_conversation_g3 SET protocol = 0;

ALTER TABLE netflow.pattern_subnet_conversation_g1 ALTER COLUMN protocol SET NOT NULL;
ALTER TABLE netflow.pattern_subnet_conversation_g2 ALTER COLUMN protocol SET NOT NULL;
ALTER TABLE netflow.pattern_subnet_conversation_g3 ALTER COLUMN protocol SET NOT NULL;

--ALTER FUNKCIJA ZA AGREGACIJU

DROP FUNCTION netflow.store_exporter_device_conversation(text, text, bigint, bigint, bigint, text, text, text, integer, text);
DROP FUNCTION netflow.store_exporter_interface_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, text, integer, text, text, integer, text);
DROP FUNCTION netflow.store_pattern_subnet_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, bigint, bigint, bigint, text, text, integer, text);
DROP FUNCTION netflow.upsert_exporter_device_conversation(text, text, bigint, bigint, bigint, inet, inet, inet, integer, timestamp without time zone);
DROP FUNCTION netflow.upsert_exporter_interface_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, inet, integer, inet, inet, integer, timestamp without time zone);
DROP FUNCTION netflow.upsert_pattern_subnet_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, integer, integer, integer, inet, inet, integer, timestamp without time zone);

CREATE OR REPLACE FUNCTION netflow.agg_exporter_device_conversation(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone)
  RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_device_conversation_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_device_conversation_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, source, destination, service, SUM(byte) AS byte, SUM(pack) AS pack, SUM(flow) AS flow, protocol FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, source, destination, service, protocol' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, source, destination, service, byte, pack, flow, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t2, srcdata.exporter, srcdata.source, srcdata.destination, srcdata.service, srcdata.byte, srcdata.pack, srcdata.flow, srcdata.protocol;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_device_conversation(text, text, text, text, timestamp without time zone, timestamp without time zone)
  OWNER TO postgres;
  
CREATE OR REPLACE FUNCTION netflow.agg_exporter_interface_conversation(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone)
  RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.exporter_interface_conversation_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.exporter_interface_conversation_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT exporter, interface, source, destination, service, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out, protocol FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY exporter, interface, source, destination, service, protocol' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, exporter, interface, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13)' USING t2, srcdata.exporter, srcdata.interface, srcdata.source, srcdata.destination, srcdata.service, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out, srcdata.protocol;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_exporter_interface_conversation(text, text, text, text, timestamp without time zone, timestamp without time zone)
  OWNER TO postgres;

CREATE OR REPLACE FUNCTION netflow.agg_pattern_subnet_conversation(srcsuffix text, srcgrain text, dstsuffix text, dstgrain text, t1 timestamp without time zone, t2 timestamp without time zone)
  RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
  srcdata RECORD;
BEGIN
  srctable := 'netflow.pattern_subnet_conversation_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.pattern_subnet_conversation_' || dstgrain || '_' || dstsuffix;
  FOR srcdata IN EXECUTE 'SELECT pattern, subnet, set, source, destination, service, SUM(byte_in) AS byte_in, SUM(byte_out) AS byte_out, SUM(pack_in) AS pack_in, SUM(pack_out) AS pack_out, SUM(flow_in) AS flow_in, SUM(flow_out) AS flow_out, protocol FROM ' || srctable::regclass || ' WHERE "time" > $1 AND "time" <= $2 GROUP BY pattern, subnet, set, source, destination, service, protocol' USING t1, t2 LOOP
    EXECUTE 'INSERT INTO ' || dsttable::regclass || ' (time, pattern, subnet, set, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14)' USING t2, srcdata.pattern, srcdata.subnet, srcdata.set, srcdata.source, srcdata.destination, srcdata.service, srcdata.byte_in, srcdata.byte_out, srcdata.pack_in, srcdata.pack_out, srcdata.flow_in, srcdata.flow_out, srcdata.protocol;
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.agg_pattern_subnet_conversation(text, text, text, text, timestamp without time zone, timestamp without time zone)
  OWNER TO postgres;

CREATE OR REPLACE FUNCTION netflow.store_exporter_device_conversation(g text, s text, b bigint, p bigint, f bigint, ip text, ip_src text, ip_dst text, srv integer, t text, prot integer)
  RETURNS void AS
$BODY$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND source = $3 AND destination = $4 AND service = $5 AND protocol = $6' INTO row USING t::timestamp, ip::inet, ip_src::inet, ip_dst::inet, srv, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND source = $6 AND destination = $7 AND service = $8 AND protocol = $9' USING row.byte+b, row.pack+p, row.flow+f, t::timestamp, ip::inet, ip_src::inet, ip_dst::inet, srv, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, source, destination, service, byte, pack, flow, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t::timestamp, ip::inet, ip_src::inet, ip_dst::inet, srv, b, p, f, prot;
  END IF;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.store_exporter_device_conversation(text, text, bigint, bigint, bigint, text, text, text, integer, text, integer)
  OWNER TO postgres;

CREATE OR REPLACE FUNCTION netflow.store_exporter_interface_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip text, ifc integer, ip_src text, ip_dst text, srv integer, t text, prot integer)
  RETURNS void AS
$BODY$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND source = $4 AND destination = $5 AND service = $6 AND protocol = $7' INTO row USING t::timestamp, ip::inet, ifc, ip_src::inet, ip_dst::inet, srv, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND source = $10 AND destination = $11 AND service = $12 AND protocol = $13' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip::inet, ifc, ip_src::inet, ip_dst::inet, srv, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13)' USING t::timestamp, ip::inet, ifc, ip_src::inet, ip_dst::inet, srv, b_in, b_out, p_in, p_out, f_in, f_out, prot;
  END IF;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.store_exporter_interface_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, text, integer, text, text, integer, text, integer)
  OWNER TO postgres;
  
CREATE OR REPLACE FUNCTION netflow.store_pattern_subnet_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern bigint, subnet bigint, subnetset bigint, ip_src text, ip_dst text, srv integer, t text, prot integer)
  RETURNS void AS
$BODY$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_conversation_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND source = $2 AND destination = $3 AND service = $4 AND pattern = $5 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND protocol = $6 AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, prot;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $6 AND protocol = $7 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnet, prot;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $6 AND protocol = $7 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnetset, prot;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND source = $8 AND destination = $9 AND service = $10 AND pattern = $11 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || ' AND protocol = $12 AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, prot;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $12 AND protocol = $13 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnet, prot;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $12 AND protocol = $13 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnetset, prot;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14)' USING t::timestamp, pattern, subnet, subnetset, ip_src::inet, ip_dst::inet, srv, b_in, b_out, p_in, p_out, f_in, f_out, prot;
  END IF;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION netflow.store_pattern_subnet_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, bigint, bigint, bigint, text, text, integer, text, integer)
  OWNER TO postgres;

CREATE OR REPLACE FUNCTION netflow.upsert_exporter_device_conversation(g text, s text, b bigint, p bigint, f bigint, ip inet, ip_src inet, ip_dst inet, srv integer, t timestamp without time zone, prot integer)
  RETURNS void AS
$BODY$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND source = $3 AND destination = $4 AND service = $5  AND protocol = $6' INTO row USING t, ip, ip_src, ip_dst, srv, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND source = $6 AND destination = $7 AND service = $8 AND protocol = $9' USING row.byte+b, row.pack+p, row.flow+f, t, ip, ip_src, ip_dst, srv, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, source, destination, service, byte, pack, flow, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t, ip, ip_src, ip_dst, srv, b, p, f, prot;
  END IF;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.upsert_exporter_device_conversation(text, text, bigint, bigint, bigint, inet, inet, inet, integer, timestamp without time zone, integer)
  OWNER TO postgres;
  
CREATE OR REPLACE FUNCTION netflow.upsert_exporter_interface_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip inet, ifc integer, ip_src inet, ip_dst inet, srv integer, t timestamp without time zone, prot integer)
  RETURNS void AS
$BODY$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND source = $4 AND destination = $5 AND service = $6 AND protocol = $7' INTO row USING t, ip, ifc, ip_src, ip_dst, srv, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND source = $10 AND destination = $11 AND service = $12 AND protocol = $13' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip, ifc, ip_src, ip_dst, srv, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13)' USING t, ip, ifc, ip_src, ip_dst, srv, b_in, b_out, p_in, p_out, f_in, f_out, prot;
  END IF;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.upsert_exporter_interface_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, inet, integer, inet, inet, integer, timestamp without time zone, integer)
  OWNER TO postgres;
  
CREATE OR REPLACE FUNCTION netflow.upsert_pattern_subnet_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern integer, subnet integer, subnetset integer, ip_src inet, ip_dst inet, srv integer, t timestamp without time zone, prot integer)
  RETURNS void AS
$BODY$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_conversation_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND source = $2 AND destination = $3 AND service = $4 AND pattern = $5 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND protocol = $6 AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, ip_src, ip_dst, srv, pattern, prot;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $6 AND protocol = $7 AND set IS NULL';
   EXECUTE sel INTO row USING t, ip_src, ip_dst, srv, pattern, subnet, prot;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $6 AND protocol = $7 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, ip_src, ip_dst, srv, pattern, subnetset, prot;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND source = $8 AND destination = $9 AND service = $10 AND pattern = $11 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND protocol = $12 AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip_src, ip_dst, srv, pattern, prot;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $12 AND protocol = $13 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip_src, ip_dst, srv, pattern, subnet, prot;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $12 AND protocol = $13 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip_src, ip_dst, srv, pattern, subnetset, prot;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out, protocol) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14)' USING t, pattern, subnet, subnetset, ip_src, ip_dst, srv, b_in, b_out, p_in, p_out, f_in, f_out, prot;
  END IF;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION netflow.upsert_pattern_subnet_conversation(text, text, bigint, bigint, bigint, bigint, bigint, bigint, integer, integer, integer, inet, inet, integer, timestamp without time zone, integer)
  OWNER TO postgres;
  
CREATE OR REPLACE FUNCTION netflow.aggregate_exporter_device_conversation(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone)
  RETURNS void AS
$BODY$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_conversation_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_conversation('g2',s2,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,g1data.source,g1data.destination,g1data.service,t2, g1data.protocol);
    PERFORM netflow.upsert_exporter_device_conversation('g3',s3,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,g1data.source,g1data.destination,g1data.service,t3, g1data.protocol);
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION netflow.aggregate_exporter_device_conversation(text, text, text, timestamp without time zone, timestamp without time zone, timestamp without time zone)
  OWNER TO postgres;
  
CREATE OR REPLACE FUNCTION netflow.aggregate_exporter_interface_conversation(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone)
  RETURNS void AS
$BODY$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_conversation_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_conversation('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,g1data.source,g1data.destination,g1data.service,t2, g1data.protocol);
    PERFORM netflow.upsert_exporter_interface_conversation('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,g1data.source,g1data.destination,g1data.service,t3, g1data.protocol);
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION netflow.aggregate_exporter_interface_conversation(text, text, text, timestamp without time zone, timestamp without time zone, timestamp without time zone)
  OWNER TO postgres;
  
CREATE OR REPLACE FUNCTION netflow.aggregate_pattern_subnet_conversation(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone)
  RETURNS void AS
$BODY$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_conversation_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_conversation('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.source,g1data.destination,g1data.service,t2, g1data.protocol);
    PERFORM netflow.upsert_pattern_subnet_conversation('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.source,g1data.destination,g1data.service,t3, g1data.protocol);
  END LOOP;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION netflow.aggregate_pattern_subnet_conversation(text, text, text, timestamp without time zone, timestamp without time zone, timestamp without time zone)
  OWNER TO postgres;


INSERT INTO framework.db_version(patch_number, patch_date) VALUES(34, current_timestamp);