--Typology scheme
CREATE TABLE typology.monitor_type
(
  id_monitor_type serial NOT NULL,
  id_device_type integer,
  id_subdevice_type integer,
  name character varying(256) NOT NULL,
  description text,
  type character varying(32) NOT NULL,
  period integer NOT NULL,
  timeout integer NOT NULL,
  enabled boolean DEFAULT false,
  ordering integer,
  charted boolean DEFAULT false,
  CONSTRAINT pk_monitor_type PRIMARY KEY (id_monitor_type),
  CONSTRAINT fk_monitor_type_device_type FOREIGN KEY (id_device_type)
	REFERENCES typology.device_type (id_device_type) MATCH FULL
	ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_monitor_type_subdevice_type FOREIGN KEY (id_subdevice_type)
	REFERENCES typology.subdevice_type (id_subdevice_type) MATCH FULL
	ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE typology.resource_type
	ADD COLUMN id_monitor_type integer;
  
ALTER TABLE typology.resource_type
	ADD CONSTRAINT fk_resource_type_monitor_type FOREIGN KEY (id_monitor_type)
	REFERENCES typology.monitor_type (id_monitor_type) MATCH FULL
	ON UPDATE NO ACTION ON DELETE NO ACTION;
	
CREATE TABLE core.expression
(
  id_expression bigserial NOT NULL,
  owner_id integer,
  field_name character varying,
  field_value character varying,
  operation character varying,
  ordering integer,
  dtype character varying(30),
  CONSTRAINT pk_expression PRIMARY KEY (id_expression),
  CONSTRAINT fk_expression_expression FOREIGN KEY (owner_id)
      REFERENCES core.expression (id_expression) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE TABLE typology.resource_composite_type
(
  id_resource_composite_type serial NOT NULL,
  id_monitor_type integer NOT NULL,
  id_resource_category integer NOT NULL,
  id_expression integer NOT NULL,
  ordering integer,
  ind_system boolean DEFAULT false,
  ind_active boolean DEFAULT true,
  hidden boolean DEFAULT false,
  CONSTRAINT pk_resource_composite_type PRIMARY KEY (id_resource_composite_type),
  CONSTRAINT fk_resource_composite_type_monitor_type FOREIGN KEY (id_monitor_type)
      REFERENCES typology.monitor_type (id_monitor_type) MATCH FULL
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_resource_composite_type_resource_category FOREIGN KEY (id_resource_category)
      REFERENCES typology.resource_category (id_resource_category) MATCH FULL
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_resource_composite_type_expression FOREIGN KEY (id_expression)
      REFERENCES core.expression (id_expression) MATCH FULL
      ON UPDATE NO ACTION ON DELETE NO ACTION
);

--Device scheme
CREATE TABLE device.monitor
(
  id_monitor serial NOT NULL,
  id_monitor_type integer NOT NULL,
  id_device integer,
  id_subdevice integer,
  enabled boolean DEFAULT false,
  state character varying(256),
  last_execution_time timestamp without time zone,
  date_created timestamp without time zone,
  date_modified timestamp without time zone,
  date_removed timestamp without time zone,
  charted boolean DEFAULT false,
  dirty boolean DEFAULT false,
  CONSTRAINT pk_monitor PRIMARY KEY (id_monitor),
  CONSTRAINT fk_monitor_monitor_type FOREIGN KEY (id_monitor_type)
    REFERENCES typology.monitor_type (id_monitor_type) MATCH SIMPLE
    ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_monitor_device FOREIGN KEY (id_device)
	REFERENCES device.device (id_device) MATCH FULL
	ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_monitor_subdevice FOREIGN KEY (id_subdevice)
	REFERENCES device.subdevice (id_subdevice) MATCH FULL
	ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE device.resource
	ADD COLUMN id_monitor integer;
  
ALTER TABLE device.resource
	ADD CONSTRAINT fk_resource_monitor FOREIGN KEY (id_monitor)
	REFERENCES device.monitor (id_monitor) MATCH FULL
	ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE TABLE device.resource_composite
(
  id_resource_composite serial NOT NULL,
  id_monitor integer NOT NULL,
  id_resource_composite_type integer NOT NULL,
  value double precision,
  hidden boolean DEFAULT false,
  CONSTRAINT pk_resource_composite PRIMARY KEY (id_resource_composite),
  CONSTRAINT fk_resource_composite_monitor FOREIGN KEY (id_monitor)
      REFERENCES device.monitor (id_monitor) MATCH FULL
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_resource_composite_resource_composite_type FOREIGN KEY (id_resource_composite_type)
      REFERENCES typology.resource_composite_type (id_resource_composite_type) MATCH FULL
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT dist_resource_composite_on_monitor UNIQUE (id_monitor, id_resource_composite_type)
);

ALTER TABLE device.resource
	ADD COLUMN hidden boolean DEFAULT false;

ALTER TABLE typology.resource_type
	ADD COLUMN hidden boolean DEFAULT false;
	
ALTER TABLE device.device
	ADD COLUMN id_primary_policy integer;
	
	

CREATE SCHEMA nms;

CREATE TABLE nms.dataset_resource_g0
(
  id_resource bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);

CREATE TABLE nms.dataset_resourcecomposite_g0
(
  id_resource_composite bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);

CREATE TABLE nms.dataset_resource_g1
(
  id_resource bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);


CREATE TABLE nms.dataset_resourcecomposite_g1
(
  id_resource_composite bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);


CREATE TABLE nms.dataset_resource_g2
(
  id_resource bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);


CREATE TABLE nms.dataset_resourcecomposite_g2
(
  id_resource_composite bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);


CREATE TABLE nms.dataset_resource_g3
(
  id_resource bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);


CREATE TABLE nms.dataset_resourcecomposite_g3
(
  id_resource_composite bigint,
  value double precision,
  "time" timestamp without time zone NOT NULL
)
WITH (
  OIDS=FALSE
);

  
CREATE INDEX idx_dataset_resource_g0 ON nms.dataset_resource_g0 (id_resource, "time");

CREATE INDEX idx_dataset_resource_g1 ON nms.dataset_resource_g1 (id_resource, "time");

CREATE INDEX idx_dataset_resource_g2 ON nms.dataset_resource_g2 (id_resource, "time");

CREATE INDEX idx_dataset_resource_g3 ON nms.dataset_resource_g3 (id_resource, "time");

CREATE INDEX idx_dataset_resourcecomposite_g0 ON nms.dataset_resourcecomposite_g0 (id_resource_composite, "time");

CREATE INDEX idx_dataset_resourcecomposite_g1 ON nms.dataset_resourcecomposite_g1 (id_resource_composite, "time");

CREATE INDEX idx_dataset_resourcecomposite_g2 ON nms.dataset_resourcecomposite_g2 (id_resource_composite, "time");

CREATE INDEX idx_dataset_resourcecomposite_g3 ON nms.dataset_resourcecomposite_g3 (id_resource_composite, "time");


INSERT INTO typology.value_type (id_value_type, name, ind_evid, ind_resource) VALUES (6, 'Counter', false, true);
INSERT INTO typology.value_type (id_value_type, name, ind_evid, ind_resource) VALUES (7, 'Counter64', false, true);

ALTER TABLE device.resource 
	ADD COLUMN snmp_value bigint;

-- UPDATE 09.06.2015 ALARMI
CREATE SCHEMA nms_conf;

CREATE TABLE nms_conf.alarm
(
  id bigserial NOT NULL,
  name character varying(100),
  message text,
  level integer,
  id_monitor_type integer NOT NULL,
  scopetype character varying(20),
  scopeid character varying(100),
  criterion bigint NOT NULL,
  CONSTRAINT pk_alarm PRIMARY KEY (id),
  CONSTRAINT fk_alarm_monitor_type FOREIGN KEY (id_monitor_type)
	  REFERENCES typology.monitor_type (id_monitor_type) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_alarm_criterion FOREIGN KEY (criterion)
      REFERENCES core.criterion (id) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE
);


CREATE TABLE nms_conf.alarm_user
(
  id_alarm bigint NOT NULL,
  id_user integer NOT NULL,
  CONSTRAINT fk_alarm_user_alarm FOREIGN KEY (id_alarm)
      REFERENCES nms_conf.alarm (id) MATCH FULL
      ON UPDATE NO ACTION ON DELETE CASCADE,
  CONSTRAINT fk_alarm_user_user FOREIGN KEY (id_user)
      REFERENCES login."user" (id_role) MATCH FULL
      ON UPDATE NO ACTION ON DELETE CASCADE
);

	
INSERT INTO framework.db_version(patch_number, patch_date) VALUES(46, current_timestamp);