INSERT INTO public.system_property 
(name, tag, type, value) 
VALUES
('READING_PERIOD_WEEKS', 'nms', 'INTEGER', '1'),
('5_MIN_WEEKS', 'nms', 'INTEGER', '2'),
('30_MIN_WEEKS', 'nms', 'INTEGER', '8'),
('3_HOUR_WEEKS', 'nms', 'INTEGER', '52'),
('DATABASE_SIZE_GB', 'nms', 'DOUBLE', '20.0');

-- UPDATE 25.09
CREATE TABLE nms.performance
(
  "time" timestamp without time zone NOT NULL,
  load_monitors int,
  monitor_count int, -- only enabled is counted
  resource_count int, -- only visible is counted
  resource_composite_count int, -- only visible is counted
  execution int,
  persistance int,
  alarm int,
  aggregation int,
  email int,
  heap bigint
)
WITH (
  OIDS=FALSE
);

CREATE INDEX performance_time_idx
  ON nms.performance
  USING btree
  ("time");
-- END OF UPDATE 25.09

-- UPDATE 28.09
DO $$DECLARE 
  tabela varchar;
BEGIN
  FOR tabela IN (SELECT table_schema || '.' || table_name 
		FROM information_schema.tables 
		WHERE table_type = 'BASE TABLE' AND table_schema NOT IN ('pg_catalog', 'information_schema')
		ORDER BY table_schema, table_name) 
  LOOP
	 EXECUTE 'ALTER TABLE ' || tabela || ' OWNER TO netvizura';
  END LOOP;
END$$;

CREATE SCHEMA multilog_conf
  AUTHORIZATION netvizura;  

ALTER TABLE multilog.alarm
    SET SCHEMA multilog_conf;

ALTER TABLE multilog.alarm_user
    SET SCHEMA multilog_conf;

CREATE SCHEMA netflow_conf
  AUTHORIZATION netvizura;

ALTER TABLE netflow.alarm
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.alarm_user
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.counter_config
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.default_config
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.pattern
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.pattern_criteria
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.sample_ratio
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.subnet
    SET SCHEMA netflow_conf;

ALTER TABLE netflow.subnet_set
    SET SCHEMA netflow_conf;

ALTER TABLE core.favorite
    SET SCHEMA users;

ALTER TABLE core.permission_module
    SET SCHEMA users;

ALTER TABLE login.principal
    SET SCHEMA users;

ALTER TABLE login.user
    SET SCHEMA users;

ALTER TABLE login.usergroup
    SET SCHEMA users;

ALTER TABLE login.usergroup_user
    SET SCHEMA users;

ALTER TABLE public.user_property
    SET SCHEMA users;

ALTER TABLE public.system_property
    SET SCHEMA framework;

ALTER TABLE core.cip_sys_module
    SET SCHEMA framework;

ALTER TABLE core.criterion
    SET SCHEMA filter;

ALTER TABLE core.expression
    SET SCHEMA filter;


DROP TABLE core.cip_sys_address_type;

DROP TABLE core.cip_usr_looking_glass_command;

DROP TABLE core.image;

DROP TABLE framework.module;

DROP TABLE view.bookmark_item;

DROP SCHEMA chart;

DROP SCHEMA core;

DROP SCHEMA "group";

DROP SCHEMA login;

DROP SCHEMA monitor;

DROP SCHEMA node;

DROP SCHEMA others;

DROP SCHEMA permission;

DROP SCHEMA view;

ALTER TABLE framework.cip_sys_module RENAME TO module;
 
DO $$DECLARE 
  shema varchar;
BEGIN
  FOR shema IN (SELECT DISTINCT table_schema
		FROM information_schema.tables 
		WHERE table_type = 'BASE TABLE' AND table_schema NOT IN ('pg_catalog', 'information_schema')
		ORDER BY table_schema) 
  LOOP
	 EXECUTE 'ALTER SCHEMA ' || shema || ' OWNER TO netvizura';
  END LOOP;
END$$;

ALTER SCHEMA public OWNER TO netvizura; -- prazna je sema pa gornja komanda je nece obuhvatiti
-- END OF UPDATE 28.09

-- UPDATE 07.10
CREATE TABLE netflow_conf.report
(
	id_report bigserial,
	name character varying(100) NOT NULL,
	period character varying(20), -- DAILY/WEEKLY/MONTHLY/QUATERLY
	email_subject character varying(100),
	email_body text,
	recipients text,
	element character varying(20), -- EXPORTER/INTERFACE/PATTERN_SUBNET
	view_type character varying(20), -- PATTERN_OVERVIEW/EXPORTER_OVERVIEW/RAW_DATA/EXPORTER_VIEW/PATTERN_VIEW/SET_VIEW/SYSTEM_OVERVIEW
	object_type character varying(20), -- AS/CONVERSATION/HOST/PROTOCOL/QOS/SERVICE/TOTAL
	chart_type character varying(20), -- THROUGHPUT/VOLUME
	chart_renderer character varying(20), -- STACK_AREA/SAMPLED_STACK_AREA/LINE/SAMPLED_LINE
	sample_count integer, -- broj sample-ova na chartu
	field_selector character varying(100), -- BYTE/PACKET/FLOW + OUT/IN/SRC/DST
	device bigint,
	interface integer,
	pattern bigint,
	subnet bigint,
	subnet_set bigint,
	CONSTRAINT pk_report PRIMARY KEY (id_report),
	CONSTRAINT report_fk_exporter FOREIGN KEY (device)
		REFERENCES device.device (id_device) MATCH SIMPLE
		ON UPDATE NO ACTION ON DELETE CASCADE,
	CONSTRAINT report_fk_pattern FOREIGN KEY (pattern)
		REFERENCES netflow_conf.pattern (id) MATCH SIMPLE
		ON UPDATE NO ACTION ON DELETE CASCADE,
	CONSTRAINT report_fk_subnet FOREIGN KEY (subnet)
		REFERENCES netflow_conf.subnet (id) MATCH SIMPLE
		ON UPDATE NO ACTION ON DELETE CASCADE,
	CONSTRAINT report_fk_subnet_set FOREIGN KEY (subnet_set)
		REFERENCES netflow_conf.subnet_set (id) MATCH SIMPLE
		ON UPDATE NO ACTION ON DELETE CASCADE
);
-- END OF UPDATE 07.10
	
INSERT INTO framework.db_version(patch_number, patch_date) VALUES (50, current_timestamp);