-- UPDATE 27.06.2016.
UPDATE framework.module SET ordering = ordering + 10;
-- END UPDATE 27.06.2016.


-- UPDATE 04.07.2016.
ALTER TABLE netflow_conf.pattern
   ADD COLUMN system boolean NOT NULL DEFAULT false;


DO $$
BEGIN
	IF EXISTS (SELECT 1 FROM netflow_conf.pattern WHERE "name" = 'All Traffic')
	THEN

		UPDATE netflow_conf.pattern
		SET 
			"name" = 'All Traffic OLD',
			description = description || ' (your setup from older NetVizura version)'
		WHERE "name" = 'All Traffic';
		
	END IF;
END $$;
   
INSERT INTO netflow_conf.pattern ("name", description, "system")
	VALUES ('All Traffic', 'Total network traffic (provided by default)', TRUE);

DO $$
	DECLARE pattern_id bigint;
BEGIN

	SELECT INTO pattern_id id FROM netflow_conf.pattern WHERE "name" = 'All Traffic';
	
	INSERT INTO netflow_conf.pattern_criteria (pattern_id, "type", "value")
	VALUES (pattern_id, 'ADDRESS', '10.0.0.0/8;172.16.0.0/12;192.168.0.0/16#%#b');

END $$;

UPDATE wizard.step
SET "type" = 'SYSTEM_PATTERN_CHANGED_CHECK'
WHERE "type" = 'PATTERN_EXISTS_CHECK';

-- END UPDATE 04.07.2016.

-- UPDATE 06.07.2016.
UPDATE wizard."step" SET name = 'Introducing Traffic Patterns' WHERE step_content = 'pattern';
-- END OF UPDATE 06.07.2016.

-- UPDATE 21.07.2016.

CREATE OR REPLACE FUNCTION netflow.bulk_agg_user_mapping_host_top_n(
    srcsuffix text,
    srcgrain text,
    dstsuffix text,
    dstgrain text,
    t1 timestamp without time zone,
    t2 timestamp without time zone,
    topn integer)
  RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
BEGIN
  srctable := 'netflow.user_mapping_host_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.user_mapping_host_' || dstgrain || '_' || dstsuffix;
  EXECUTE 'CREATE TEMP TABLE host_kes
	ON COMMIT DROP
	AS
	SELECT $1, user_id, host, SUM(byte_upload) AS byte_upload, SUM(byte_download) AS byte_download, SUM(pack_upload) AS pack_upload, SUM(pack_download) AS pack_download, SUM(flow_upload) AS flow_upload, SUM(flow_download) AS flow_download
	FROM ' || srctable::regclass || '
	WHERE "time" > $1 AND "time" <= $2 
	GROUP BY user_id, host
	ORDER BY user_id;
	'
USING t1, t2;

CREATE INDEX host_kes_index_on_user_id ON host_kes USING hash (user_id);

EXECUTE 'INSERT INTO ' || dsttable::regclass || ' 
	(time, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download) 
	(
		SELECT 
			$2, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download  
		FROM (
			select 
				user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download,  
				row_number() OVER(PARTITION BY user_id ORDER BY byte_upload DESC) <= $1 as rnk
			from host_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download  
		FROM (
			select 
				user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download, 
				row_number() OVER(PARTITION BY user_id ORDER BY byte_download DESC) <= $1 as rnk
			from host_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download  
		FROM (
			select 
				user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download,  
				row_number() OVER(PARTITION BY user_id ORDER BY pack_upload DESC) <= $1 as rnk
			from host_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download  
		FROM (
			select 
				user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download,  
				row_number() OVER(PARTITION BY user_id ORDER BY pack_download DESC) <= $1 as rnk
			from host_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download  
		FROM (
			select 
				user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download, 
				row_number() OVER(PARTITION BY user_id ORDER BY flow_upload DESC) <= $1 as rnk
			from host_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download  
		FROM (
			select 
				$2, user_id, host, byte_upload, byte_download, pack_upload, pack_download, flow_upload, flow_download,  
				row_number() OVER(PARTITION BY user_id ORDER BY flow_download DESC) <= $1 as rnk
			from host_kes
			order by rnk
		) sub
		where rnk = true
	)'
USING topn, t2;

END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.bulk_agg_user_mapping_host_top_n(text, text, text, text, timestamp without time zone, timestamp without time zone, integer)
  OWNER TO postgres;

CREATE OR REPLACE FUNCTION netflow.bulk_agg_user_mapping_conversation_top_n(
    srcsuffix text,
    srcgrain text,
    dstsuffix text,
    dstgrain text,
    t1 timestamp without time zone,
    t2 timestamp without time zone,
    topn integer)
  RETURNS void AS
$BODY$
DECLARE
  srctable TEXT;
  dsttable TEXT;
BEGIN
  srctable := 'netflow.user_mapping_conversation_' || srcgrain || '_' || srcsuffix;
  dsttable := 'netflow.user_mapping_conversation_' || dstgrain || '_' || dstsuffix;
  EXECUTE 'CREATE TEMP TABLE conv_kes
	ON COMMIT DROP
	AS
	SELECT $1, user_id, source, destination, service, protocol, SUM(byte_download) AS byte_download, SUM(byte_upload) AS byte_upload, SUM(pack_download) AS pack_download, SUM(pack_upload) AS pack_upload, SUM(flow_download) AS flow_download, SUM(flow_upload) AS flow_upload
	FROM ' || srctable::regclass || '
	WHERE "time" > $1 AND "time" <= $2 
	GROUP BY user_id, source, destination, service, protocol
	ORDER BY user_id;
	'
USING t1, t2;

CREATE INDEX conv_kes_index_on_user_id ON conv_kes USING hash (user_id);

EXECUTE 'INSERT INTO ' || dsttable::regclass || ' 
	(time, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload) 
	(
		SELECT 
			$2, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload  
		FROM (
			select 
				user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload,  
				row_number() OVER(PARTITION BY user_id ORDER BY byte_upload DESC) <= $1 as rnk
			from conv_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload  
		FROM (
			select 
				user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload, 
				row_number() OVER(PARTITION BY user_id ORDER BY byte_download DESC) <= $1 as rnk
			from conv_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload  
		FROM (
			select 
				user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload,  
				row_number() OVER(PARTITION BY user_id ORDER BY pack_upload DESC) <= $1 as rnk
			from conv_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload  
		FROM (
			select 
				user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload,  
				row_number() OVER(PARTITION BY user_id ORDER BY pack_download DESC) <= $1 as rnk
			from conv_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload  
		FROM (
			select 
				user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload, 
				row_number() OVER(PARTITION BY user_id ORDER BY flow_upload DESC) <= $1 as rnk
			from conv_kes
			order by rnk
		) sub
		where rnk = true
	) UNION  
	(
		SELECT 
			$2, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload  
		FROM (
			select 
				$2, user_id, source, destination, service, protocol, byte_download, byte_upload, pack_download, pack_upload, flow_download, flow_upload,  
				row_number() OVER(PARTITION BY user_id ORDER BY flow_download DESC) <= $1 as rnk
			from conv_kes
			order by rnk
		) sub
		where rnk = true
	)'
USING topn, t2;

END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.bulk_agg_user_mapping_conversation_top_n(text, text, text, text, timestamp without time zone, timestamp without time zone, integer)
  OWNER TO postgres;

  
CREATE OR REPLACE FUNCTION netflow.bulk_aggregate_top_n(
    srcsuffix text,
    srcgrain text,
    dstsuffix text,
    dstgrain text,
    t1 timestamp without time zone,
    t2 timestamp without time zone,
    topn integer)
  RETURNS void AS
$BODY$
DECLARE
BEGIN
  PERFORM netflow.bulk_agg_exporter_device_total(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_device_as(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_device_conversation(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_device_host(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_device_protocol(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_device_qos(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_device_service(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);

  PERFORM netflow.bulk_agg_exporter_interface_total(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_interface_as(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_interface_conversation(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_interface_host(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_interface_protocol(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_interface_qos(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_exporter_interface_service(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);

  PERFORM netflow.bulk_agg_pattern_subnet_total(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_pattern_subnet_as(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_pattern_subnet_conversation(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_pattern_subnet_host(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_pattern_subnet_protocol(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_pattern_subnet_qos(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_pattern_subnet_service(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);

  PERFORM netflow.bulk_agg_user_mapping_total(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_user_mapping_host_top_n(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2,topn);
  PERFORM netflow.bulk_agg_user_mapping_service(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_user_mapping_protocol(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_user_mapping_as(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_user_mapping_qos(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2);
  PERFORM netflow.bulk_agg_user_mapping_conversation_top_n(srcsuffix,srcgrain,dstsuffix,dstgrain,t1,t2,topn);
END;
$BODY$
  LANGUAGE plpgsql VOLATILE STRICT
  COST 100;
ALTER FUNCTION netflow.bulk_aggregate_top_n(text, text, text, text, timestamp without time zone, timestamp without time zone, integer)
  OWNER TO postgres; 
-- END OF UPDATE 21.07.2016.

-- UPDATE 27.07.2016.
  ALTER TABLE netflow.user_mapping_total_g1 DROP CONSTRAINT user_mapping_total_g1_fkey;
  ALTER TABLE netflow.user_mapping_total_g2 DROP CONSTRAINT user_mapping_total_g2_fkey;
  ALTER TABLE netflow.user_mapping_total_g3 DROP CONSTRAINT user_mapping_total_g3_fkey;
-- END OF UPDATE 27.07.2016.

INSERT INTO framework.db_version(patch_number, patch_date) VALUES (57, current_timestamp);