SET statement_timeout = 0;
SET lock_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;

CREATE DATABASE netvizura ENCODING = 'UTF8';

DROP USER IF EXISTS netvizura;
CREATE USER netvizura PASSWORD 'netvizura';
ALTER USER netvizura SUPERUSER;

\connect netvizura

SET statement_timeout = 0;
SET lock_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;

CREATE SCHEMA chart;

CREATE SCHEMA core;

CREATE SCHEMA device;

CREATE SCHEMA event_alarm_action;

CREATE SCHEMA filter;

CREATE SCHEMA flow;

CREATE SCHEMA framework;

CREATE SCHEMA "group";

CREATE SCHEMA login;

CREATE SCHEMA mapping;

CREATE SCHEMA mib;

CREATE SCHEMA monitor;

CREATE SCHEMA multilog;

CREATE SCHEMA netflow;

CREATE SCHEMA node;

CREATE SCHEMA others;

CREATE SCHEMA permission;

CREATE SCHEMA policy;

CREATE SCHEMA report;

CREATE SCHEMA typology;

CREATE SCHEMA users;

CREATE SCHEMA view;

CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;

COMMENT ON EXTENSION plpgsql IS 'PL/pgSQL procedural language';

SET search_path = netflow, pg_catalog;

CREATE FUNCTION aggregate_all(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $$
DECLARE
BEGIN
	PERFORM netflow.aggregate_exporter_device_total(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_device_as(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_device_conversation(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_device_host(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_device_protocol(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_device_qos(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_device_service(s1,s2,s3,t1,t2,t3);

	PERFORM netflow.aggregate_exporter_interface_total(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_interface_as(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_interface_conversation(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_interface_host(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_interface_protocol(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_interface_qos(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_exporter_interface_service(s1,s2,s3,t1,t2,t3);

	PERFORM netflow.aggregate_pattern_subnet_total(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_pattern_subnet_as(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_pattern_subnet_conversation(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_pattern_subnet_host(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_pattern_subnet_protocol(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_pattern_subnet_qos(s1,s2,s3,t1,t2,t3);
	PERFORM netflow.aggregate_pattern_subnet_service(s1,s2,s3,t1,t2,t3);
END;
$$;

CREATE FUNCTION aggregate_exporter_device_as(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_as_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_as('g2',s2,g1data.byte_src,g1data.byte_dst,g1data.pack_src,g1data.pack_dst,g1data.flow_src,g1data.flow_dst,g1data.as,g1data.exporter,t2);
    PERFORM netflow.upsert_exporter_device_as('g3',s3,g1data.byte_src,g1data.byte_dst,g1data.pack_src,g1data.pack_dst,g1data.flow_src,g1data.flow_dst,g1data.as,g1data.exporter,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_device_conversation(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_conversation_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_conversation('g2',s2,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,g1data.source,g1data.destination,g1data.service,t2);
    PERFORM netflow.upsert_exporter_device_conversation('g3',s3,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,g1data.source,g1data.destination,g1data.service,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_device_host(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_host_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_host('g2',s2,g1data.byte_src,g1data.byte_dst,g1data.pack_src,g1data.pack_dst,g1data.flow_src,g1data.flow_dst,g1data.host,g1data.exporter,t2);
    PERFORM netflow.upsert_exporter_device_host('g3',s3,g1data.byte_src,g1data.byte_dst,g1data.pack_src,g1data.pack_dst,g1data.flow_src,g1data.flow_dst,g1data.host,g1data.exporter,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_device_protocol(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_protocol_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_protocol('g2',s2,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,t2,g1data.protocol);
    PERFORM netflow.upsert_exporter_device_protocol('g3',s3,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,t3,g1data.protocol);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_device_qos(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_qos_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_qos('g2',s2,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,t2,g1data.tos);
    PERFORM netflow.upsert_exporter_device_qos('g3',s3,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,t3,g1data.tos);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_device_service(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_service_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_service('g2',s2,g1data.byte_src,g1data.byte_dst,g1data.pack_src,g1data.pack_dst,g1data.flow_src,g1data.flow_dst,g1data.port,g1data.exporter,t2);
    PERFORM netflow.upsert_exporter_device_service('g3',s3,g1data.byte_src,g1data.byte_dst,g1data.pack_src,g1data.pack_dst,g1data.flow_src,g1data.flow_dst,g1data.port,g1data.exporter,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_device_total(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_device_total_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_device_total('g2',s2,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,t2);
    PERFORM netflow.upsert_exporter_device_total('g3',s3,g1data.byte,g1data.pack,g1data.flow,g1data.exporter,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_interface_as(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_as_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_as('g2',s2,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.as,g1data.exporter,g1data.interface,t2);
    PERFORM netflow.upsert_exporter_interface_as('g3',s3,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.as,g1data.exporter,g1data.interface,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_interface_conversation(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_conversation_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_conversation('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,g1data.source,g1data.destination,g1data.service,t2);
    PERFORM netflow.upsert_exporter_interface_conversation('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,g1data.source,g1data.destination,g1data.service,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_interface_host(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_host_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_host('g2',s2,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.host,g1data.exporter,g1data.interface,t2);
    PERFORM netflow.upsert_exporter_interface_host('g3',s3,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.host,g1data.exporter,g1data.interface,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_interface_protocol(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_protocol_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_protocol('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,t2,g1data.protocol);
    PERFORM netflow.upsert_exporter_interface_protocol('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,t3,g1data.protocol);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_interface_qos(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_qos_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_qos('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,t2,g1data.tos);
    PERFORM netflow.upsert_exporter_interface_qos('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,t3,g1data.tos);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_interface_service(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_service_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_service('g2',s2,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.port,g1data.exporter,g1data.interface,t2);
    PERFORM netflow.upsert_exporter_interface_service('g3',s3,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.port,g1data.exporter,g1data.interface,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_exporter_interface_total(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.exporter_interface_total_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_exporter_interface_total('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,t2);
    PERFORM netflow.upsert_exporter_interface_total('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.exporter,g1data.interface,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_pattern_subnet_as(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_as_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_as('g2',s2,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.pattern,g1data.subnet,g1data.set,g1data.as,t2);
    PERFORM netflow.upsert_pattern_subnet_as('g3',s3,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.pattern,g1data.subnet,g1data.set,g1data.as,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_pattern_subnet_conversation(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_conversation_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_conversation('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.source,g1data.destination,g1data.service,t2);
    PERFORM netflow.upsert_pattern_subnet_conversation('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.source,g1data.destination,g1data.service,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_pattern_subnet_host(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_host_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_host('g2',s2,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.pattern,g1data.subnet,g1data.set,g1data.host,t2);
    PERFORM netflow.upsert_pattern_subnet_host('g3',s3,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.pattern,g1data.subnet,g1data.set,g1data.host,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_pattern_subnet_protocol(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_protocol_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_protocol('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.protocol,t2);
    PERFORM netflow.upsert_pattern_subnet_protocol('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.protocol,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_pattern_subnet_qos(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_qos_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_qos('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.tos,t2);
    PERFORM netflow.upsert_pattern_subnet_qos('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,g1data.tos,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_pattern_subnet_service(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_service_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_service('g2',s2,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.pattern,g1data.subnet,g1data.set,g1data.port,t2);
    PERFORM netflow.upsert_pattern_subnet_service('g3',s3,g1data.byte_in_src,g1data.byte_in_dst,g1data.byte_out_src,g1data.byte_out_dst,g1data.pack_in_src,g1data.pack_in_dst,g1data.pack_out_src,g1data.pack_out_dst,g1data.flow_in_src,g1data.flow_in_dst,g1data.flow_out_src,g1data.flow_out_dst,g1data.pattern,g1data.subnet,g1data.set,g1data.port,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION aggregate_pattern_subnet_total(s1 text, s2 text, s3 text, t1 timestamp without time zone, t2 timestamp without time zone, t3 timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  g1table TEXT;
  g1data RECORD;
BEGIN
  g1table := 'netflow.pattern_subnet_total_g1_' || s1;
  FOR g1data IN EXECUTE 'SELECT * FROM ' || g1table::regclass || ' WHERE time = $1' USING t1 LOOP
    PERFORM netflow.upsert_pattern_subnet_total('g2',s2,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,t2);
    PERFORM netflow.upsert_pattern_subnet_total('g3',s3,g1data.byte_in,g1data.byte_out,g1data.pack_in,g1data.pack_out,g1data.flow_in,g1data.flow_out,g1data.pattern,g1data.subnet,g1data.set,t3);
  END LOOP;
END;
$_$;

CREATE FUNCTION store_exporter_device_as(g text, s text, b_src bigint, b_dst bigint, p_src bigint, p_dst bigint, f_src bigint, f_dst bigint, a integer, ip text, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_as_' || g || '_' || s;
  EXECUTE 'SELECT byte_src,byte_dst,pack_src,pack_dst,flow_src,flow_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND "as" = $3' INTO row USING t::timestamp, ip::inet, a;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_src = $1, byte_dst = $2, pack_src = $3, pack_dst = $4, flow_src = $5, flow_dst = $6 WHERE time = $7 AND exporter = $8 AND "as" = $9' USING row.byte_src+b_src, row.byte_dst+b_dst, row.pack_src+p_src, row.pack_dst+p_dst, row.flow_src+f_src, row.flow_dst+f_dst, t::timestamp, ip::inet, a;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, "as", byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t::timestamp, ip::inet, a, b_src, b_dst, p_src, p_dst, f_src, f_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_device_conversation(g text, s text, b bigint, p bigint, f bigint, ip text, ip_src text, ip_dst text, srv integer, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND source = $3 AND destination = $4 AND service = $5' INTO row USING t::timestamp, ip::inet, ip_src::inet, ip_dst::inet, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND source = $6 AND destination = $7 AND service = $8' USING row.byte+b, row.pack+p, row.flow+f, t::timestamp, ip::inet, ip_src::inet, ip_dst::inet, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, source, destination, service, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6, $7, $8)' USING t::timestamp, ip::inet, ip_src::inet, ip_dst::inet, srv, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_device_host(g text, s text, b_src bigint, b_dst bigint, p_src bigint, p_dst bigint, f_src bigint, f_dst bigint, h text, ip text, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_host_' || g || '_' || s;
  EXECUTE 'SELECT byte_src,byte_dst,pack_src,pack_dst,flow_src,flow_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND host = $3' INTO row USING t::timestamp, ip::inet, h::inet;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_src = $1, byte_dst = $2, pack_src = $3, pack_dst = $4, flow_src = $5, flow_dst = $6 WHERE time = $7 AND exporter = $8 AND host = $9' USING row.byte_src+b_src, row.byte_dst+b_dst, row.pack_src+p_src, row.pack_dst+p_dst, row.flow_src+f_src, row.flow_dst+f_dst, t::timestamp, ip::inet, h::inet;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, host, byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t::timestamp, ip::inet, h::inet, b_src, b_dst, p_src, p_dst, f_src, f_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_device_protocol(g text, s text, b bigint, p bigint, f bigint, ip text, t text, prot integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_protocol_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND protocol = $3' INTO row USING t::timestamp, ip::inet, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND protocol = $6' USING row.byte+b, row.pack+p, row.flow+f, t::timestamp, ip::inet, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, protocol, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6)' USING t::timestamp, ip::inet, prot, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_device_qos(g text, s text, b bigint, p bigint, f bigint, ip text, t text, tos integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_qos_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND tos = $3' INTO row USING t::timestamp, ip::inet, tos;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND tos = $6' USING row.byte+b, row.pack+p, row.flow+f, t::timestamp, ip::inet, tos;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, tos, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6)' USING t::timestamp, ip::inet, tos, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_device_service(g text, s text, b_src bigint, b_dst bigint, p_src bigint, p_dst bigint, f_src bigint, f_dst bigint, srv integer, ip text, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_service_' || g || '_' || s;
  EXECUTE 'SELECT byte_src,byte_dst,pack_src,pack_dst,flow_src,flow_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND port = $3' INTO row USING t::timestamp, ip::inet, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_src = $1, byte_dst = $2, pack_src = $3, pack_dst = $4, flow_src = $5, flow_dst = $6 WHERE time = $7 AND exporter = $8 AND port = $9' USING row.byte_src+b_src, row.byte_dst+b_dst, row.pack_src+p_src, row.pack_dst+p_dst, row.flow_src+f_src, row.flow_dst+f_dst, t::timestamp, ip::inet, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, port, byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t::timestamp, ip::inet, srv, b_src, b_dst, p_src, p_dst, f_src, f_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_device_total(g text, s text, b bigint, p bigint, f bigint, ip text, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_total_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2' INTO row USING t::timestamp, ip::inet;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5' USING row.byte+b, row.pack+p, row.flow+f, t::timestamp, ip::inet;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, byte, pack, flow) VALUES($1, $2, $3, $4, $5)' USING t::timestamp, ip::inet, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_interface_as(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, a integer, ip text, ifc integer, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_as_' || g || '_' || s;
  EXECUTE 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND "as" = $4' INTO row USING t::timestamp, ip::inet, ifc, a;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND exporter = $14 AND interface = $15 AND "as" = $16' USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, ip::inet, ifc, a;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, "as", byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t::timestamp, ip::inet, ifc, a, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_interface_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip text, ifc integer, ip_src text, ip_dst text, srv integer, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND source = $4 AND destination = $5 AND service = $6' INTO row USING t::timestamp, ip::inet, ifc, ip_src::inet, ip_dst::inet, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND source = $10 AND destination = $11 AND service = $12' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip::inet, ifc, ip_src::inet, ip_dst::inet, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12)' USING t::timestamp, ip::inet, ifc, ip_src::inet, ip_dst::inet, srv, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_interface_host(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, h text, ip text, ifc integer, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_host_' || g || '_' || s;
  EXECUTE 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND host = $4' INTO row USING t::timestamp, ip::inet, ifc, h::inet;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND exporter = $14 AND interface = $15 AND host = $16' USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, ip::inet, ifc, h::inet;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, host, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t::timestamp, ip::inet, ifc, h::inet, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_interface_protocol(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip text, ifc integer, t text, prot integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_protocol_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND protocol = $4' INTO row USING t::timestamp, ip::inet, ifc, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND protocol = $10' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip::inet, ifc, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, protocol, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t::timestamp, ip::inet, ifc, prot, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_interface_qos(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip text, ifc integer, t text, tos integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_qos_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND tos = $4' INTO row USING t::timestamp, ip::inet, ifc, tos;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND tos = $10' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip::inet, ifc, tos;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, tos, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t::timestamp, ip::inet, ifc, tos, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_interface_service(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, srv integer, ip text, ifc integer, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_service_' || g || '_' || s;
  EXECUTE 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND port = $4' INTO row USING t::timestamp, ip::inet, ifc, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND exporter = $14 AND interface = $15 AND port = $16' USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, ip::inet, ifc, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, port, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t::timestamp, ip::inet, ifc, srv, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_exporter_interface_total(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip text, ifc integer, t text) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_total_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3' INTO row USING t::timestamp, ip::inet, ifc;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip::inet, ifc;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t::timestamp, ip::inet, ifc, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION store_pattern_subnet_as(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, pattern bigint, subnet bigint, subnetset bigint, a integer, t text) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_as_' || g || '_' || s;
  sel := 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND "as" = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, a, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, a, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, a, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND "as" = $14 AND pattern = $15 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
	  EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, a, pattern;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $16 AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, a, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $16 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, a, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, "as", byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t::timestamp, pattern, subnet, subnetset, a, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_pattern_subnet_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern bigint, subnet bigint, subnetset bigint, ip_src text, ip_dst text, srv integer, t text) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_conversation_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND source = $2 AND destination = $3 AND service = $4 AND pattern = $5 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $6 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $6 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND source = $8 AND destination = $9 AND service = $10 AND pattern = $11 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $12 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $12 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, ip_src::inet, ip_dst::inet, srv, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13)' USING t::timestamp, pattern, subnet, subnetset, ip_src::inet, ip_dst::inet, srv, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION store_pattern_subnet_host(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, pattern bigint, subnet bigint, subnetset bigint, h text, t text) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_host_' || g || '_' || s;
  sel := 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND host = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, h::inet, pattern;   
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, h::inet, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, h::inet, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND host = $14 AND pattern = $15 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, h::inet, pattern;	  
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $16 AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, h::inet, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $16 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, h::inet, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, host, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t::timestamp, pattern, subnet, subnetset, h::inet, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_pattern_subnet_protocol(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern bigint, subnet bigint, subnetset bigint, prot integer, t text) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_protocol_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND protocol = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, prot, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, prot, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, prot, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND protocol = $8 AND pattern = $9 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, prot, pattern;	  
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $10 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, prot, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $10 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, prot, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, protocol, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)' USING t::timestamp, pattern, subnet, subnetset, prot, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION store_pattern_subnet_qos(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern bigint, subnet bigint, subnetset bigint, tos integer, t text) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_qos_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND tos = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, tos, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, tos, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, tos, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND tos = $8 AND pattern = $9 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, tos, pattern;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $10 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, tos, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $10 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, tos, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, tos, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)' USING t::timestamp, pattern, subnet, subnetset, tos, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION store_pattern_subnet_service(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, pattern bigint, subnet bigint, subnetset bigint, srv integer, t text) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_service_' || g || '_' || s;
  sel := 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND port = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, srv, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, srv, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, srv, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND port = $14 AND pattern = $15 ';
    IF subnet IS NULL AND subnetset iS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, srv, pattern;	  
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $16 AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, srv, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $16 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t::timestamp, srv, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, port, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t::timestamp, pattern, subnet, subnetset, srv, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION store_pattern_subnet_total(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern bigint, subnet bigint, subnetset bigint, t text) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_total_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND pattern = $2 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $3 AND set IS NULL';
   EXECUTE sel INTO row USING t::timestamp, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $3 AND subnet IS NULL';
   EXECUTE sel INTO row USING t::timestamp, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND pattern = $8 ';
	IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
	  EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, pattern;
    ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $9 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $9 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t::timestamp, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t::timestamp, pattern, subnet, subnetset, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_device_as(g text, s text, b_src bigint, b_dst bigint, p_src bigint, p_dst bigint, f_src bigint, f_dst bigint, a integer, ip inet, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_as_' || g || '_' || s;
  EXECUTE 'SELECT byte_src,byte_dst,pack_src,pack_dst,flow_src,flow_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND "as" = $3' INTO row USING t, ip, a;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_src = $1, byte_dst = $2, pack_src = $3, pack_dst = $4, flow_src = $5, flow_dst = $6 WHERE time = $7 AND exporter = $8 AND "as" = $9' USING row.byte_src+b_src, row.byte_dst+b_dst, row.pack_src+p_src, row.pack_dst+p_dst, row.flow_src+f_src, row.flow_dst+f_dst, t, ip, a;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, "as", byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t, ip, a, b_src, b_dst, p_src, p_dst, f_src, f_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_device_conversation(g text, s text, b bigint, p bigint, f bigint, ip inet, ip_src inet, ip_dst inet, srv integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND source = $3 AND destination = $4 AND service = $5' INTO row USING t, ip, ip_src, ip_dst, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND source = $6 AND destination = $7 AND service = $8' USING row.byte+b, row.pack+p, row.flow+f, t, ip, ip_src, ip_dst, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, source, destination, service, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6, $7, $8)' USING t, ip, ip_src, ip_dst, srv, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_device_host(g text, s text, b_src bigint, b_dst bigint, p_src bigint, p_dst bigint, f_src bigint, f_dst bigint, h inet, ip inet, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_host_' || g || '_' || s;
  EXECUTE 'SELECT byte_src,byte_dst,pack_src,pack_dst,flow_src,flow_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND host = $3' INTO row USING t, ip, h;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_src = $1, byte_dst = $2, pack_src = $3, pack_dst = $4, flow_src = $5, flow_dst = $6 WHERE time = $7 AND exporter = $8 AND host = $9' USING row.byte_src+b_src, row.byte_dst+b_dst, row.pack_src+p_src, row.pack_dst+p_dst, row.flow_src+f_src, row.flow_dst+f_dst, t, ip, h;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, host, byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t, ip, h, b_src, b_dst, p_src, p_dst, f_src, f_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_device_protocol(g text, s text, b bigint, p bigint, f bigint, ip inet, t timestamp without time zone, prot integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_protocol_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND protocol = $3' INTO row USING t, ip, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND protocol = $6' USING row.byte+b, row.pack+p, row.flow+f, t, ip, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, protocol, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6)' USING t, ip, prot, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_device_qos(g text, s text, b bigint, p bigint, f bigint, ip inet, t timestamp without time zone, tos integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_qos_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND tos = $3' INTO row USING t, ip, tos;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5 AND tos = $6' USING row.byte+b, row.pack+p, row.flow+f, t, ip, tos;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, tos, byte, pack, flow) VALUES($1, $2, $3, $4, $5, $6)' USING t, ip, tos, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_device_service(g text, s text, b_src bigint, b_dst bigint, p_src bigint, p_dst bigint, f_src bigint, f_dst bigint, srv integer, ip inet, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_service_' || g || '_' || s;
  EXECUTE 'SELECT byte_src,byte_dst,pack_src,pack_dst,flow_src,flow_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND port = $3' INTO row USING t, ip, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_src = $1, byte_dst = $2, pack_src = $3, pack_dst = $4, flow_src = $5, flow_dst = $6 WHERE time = $7 AND exporter = $8 AND port = $9' USING row.byte_src+b_src, row.byte_dst+b_dst, row.pack_src+p_src, row.pack_dst+p_dst, row.flow_src+f_src, row.flow_dst+f_dst, t, ip, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, port, byte_src, byte_dst, pack_src, pack_dst, flow_src, flow_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t, ip, srv, b_src, b_dst, p_src, p_dst, f_src, f_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_device_total(g text, s text, b bigint, p bigint, f bigint, ip inet, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_device_total_' || g || '_' || s;
  EXECUTE 'SELECT byte,pack,flow FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2' INTO row USING t, ip;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte = $1, pack = $2, flow = $3 WHERE time = $4 AND exporter = $5' USING row.byte+b, row.pack+p, row.flow+f, t, ip;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, byte, pack, flow) VALUES($1, $2, $3, $4, $5)' USING t, ip, b, p, f;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_interface_as(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, a integer, ip inet, ifc integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_as_' || g || '_' || s;
  EXECUTE 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND "as" = $4' INTO row USING t, ip, ifc, a;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND exporter = $14 AND interface = $15 AND "as" = $16' USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, ip, ifc, a;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, "as", byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t, ip, ifc, a, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_interface_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip inet, ifc integer, ip_src inet, ip_dst inet, srv integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_conversation_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND source = $4 AND destination = $5 AND service = $6' INTO row USING t, ip, ifc, ip_src, ip_dst, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND source = $10 AND destination = $11 AND service = $12' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip, ifc, ip_src, ip_dst, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12)' USING t, ip, ifc, ip_src, ip_dst, srv, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_interface_host(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, h inet, ip inet, ifc integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_host_' || g || '_' || s;
  EXECUTE 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND host = $4' INTO row USING t, ip, ifc, h;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND exporter = $14 AND interface = $15 AND host = $16' USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, ip, ifc, h;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, host, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t, ip, ifc, h, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_interface_protocol(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip inet, ifc integer, t timestamp without time zone, prot integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_protocol_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND protocol = $4' INTO row USING t, ip, ifc, prot;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND protocol = $10' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip, ifc, prot;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, protocol, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t, ip, ifc, prot, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_interface_qos(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip inet, ifc integer, t timestamp without time zone, tos integer) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_qos_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND tos = $4' INTO row USING t, ip, ifc, tos;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9 AND tos = $10' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip, ifc, tos;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, tos, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t, ip, ifc, tos, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_interface_service(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, srv integer, ip inet, ifc integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_service_' || g || '_' || s;
  EXECUTE 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3 AND port = $4' INTO row USING t, ip, ifc, srv;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND exporter = $14 AND interface = $15 AND port = $16' USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, ip, ifc, srv;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, port, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)' USING t, ip, ifc, srv, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_exporter_interface_total(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, ip inet, ifc integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql STRICT
    AS $_$
DECLARE
  row RECORD;
  tablename text;
BEGIN
  tablename := 'netflow.exporter_interface_total_' || g || '_' || s;
  EXECUTE 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND exporter = $2 AND interface = $3' INTO row USING t, ip, ifc;
  IF row IS NOT NULL THEN
    EXECUTE 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND exporter = $8 AND interface = $9' USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip, ifc;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, exporter, interface, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9)' USING t, ip, ifc, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_pattern_subnet_as(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, pattern integer, subnet integer, subnetset integer, a integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_as_' || g || '_' || s;
  sel := 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND "as" = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, a, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t, a, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, a, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND "as" = $14 AND pattern = $15 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
	  EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, a, pattern;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $16 AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, a, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $16 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, a, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, "as", byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t, pattern, subnet, subnetset, a, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_pattern_subnet_conversation(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern integer, subnet integer, subnetset integer, ip_src inet, ip_dst inet, srv integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_conversation_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND source = $2 AND destination = $3 AND service = $4 AND pattern = $5 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, ip_src, ip_dst, srv, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $6 AND set IS NULL';
   EXECUTE sel INTO row USING t, ip_src, ip_dst, srv, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $6 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, ip_src, ip_dst, srv, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND source = $8 AND destination = $9 AND service = $10 AND pattern = $11 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip_src, ip_dst, srv, pattern;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $12 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip_src, ip_dst, srv, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $12 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, ip_src, ip_dst, srv, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, source, destination, service, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13)' USING t, pattern, subnet, subnetset, ip_src, ip_dst, srv, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_pattern_subnet_host(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, pattern integer, subnet integer, subnetset integer, h inet, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_host_' || g || '_' || s;
  sel := 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND host = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, h, pattern;   
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t, h, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, h, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND host = $14 AND pattern = $15 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, h, pattern;	  
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $16 AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, h, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $16 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, h, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, host, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t, pattern, subnet, subnetset, h, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_pattern_subnet_protocol(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern integer, subnet integer, subnetset integer, prot integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_protocol_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND protocol = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, prot, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t, prot, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, prot, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND protocol = $8 AND pattern = $9 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, prot, pattern;	  
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $10 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, prot, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $10 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, prot, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, protocol, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)' USING t, pattern, subnet, subnetset, prot, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_pattern_subnet_qos(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern integer, subnet integer, subnetset integer, tos integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_qos_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND tos = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, tos, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t, tos, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, tos, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND tos = $8 AND pattern = $9 ';
    IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, tos, pattern;
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $10 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, tos, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $10 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, tos, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, tos, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11)' USING t, pattern, subnet, subnetset, tos, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_pattern_subnet_service(g text, s text, b_in_src bigint, b_in_dst bigint, b_out_src bigint, b_out_dst bigint, p_in_src bigint, p_in_dst bigint, p_out_src bigint, p_out_dst bigint, f_in_src bigint, f_in_dst bigint, f_out_src bigint, f_out_dst bigint, pattern integer, subnet integer, subnetset integer, srv integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_service_' || g || '_' || s;
  sel := 'SELECT byte_in_src,byte_in_dst,byte_out_src,byte_out_dst,pack_in_src,pack_in_dst,pack_out_src,pack_out_dst,flow_in_src,flow_in_dst,flow_out_src,flow_out_dst FROM ' || tablename::regclass || ' WHERE time = $1 AND port = $2 AND pattern = $3 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, srv, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $4 AND set IS NULL';
   EXECUTE sel INTO row USING t, srv, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $4 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, srv, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in_src = $1, byte_in_dst = $2, byte_out_src = $3, byte_out_dst = $4, pack_in_src = $5, pack_in_dst = $6, pack_out_src = $7, pack_out_dst = $8, flow_in_src = $9, flow_in_dst = $10, flow_out_src = $11, flow_out_dst = $12 WHERE time = $13 AND port = $14 AND pattern = $15 ';
    IF subnet IS NULL AND subnetset iS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, srv, pattern;	  
	ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $16 AND set IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, srv, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $16 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in_src+b_in_src, row.byte_in_dst+b_in_dst, row.byte_out_src+b_out_src, row.byte_out_dst+b_out_dst, row.pack_in_src+p_in_src, row.pack_in_dst+p_in_dst, row.pack_out_src+p_out_src, row.pack_out_dst+p_out_dst, row.flow_in_src+f_in_src, row.flow_in_dst+f_in_dst, row.flow_out_src+f_out_src, row.flow_out_dst+f_out_dst, t, srv, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, port, byte_in_src, byte_in_dst, byte_out_src, byte_out_dst, pack_in_src, pack_in_dst, pack_out_src, pack_out_dst, flow_in_src, flow_in_dst, flow_out_src, flow_out_dst) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16, $17)' USING t, pattern, subnet, subnetset, srv, b_in_src, b_in_dst, b_out_src, b_out_dst, p_in_src, p_in_dst, p_out_src, p_out_dst, f_in_src, f_in_dst, f_out_src, f_out_dst;
  END IF;
END;
$_$;

CREATE FUNCTION upsert_pattern_subnet_total(g text, s text, b_in bigint, b_out bigint, p_in bigint, p_out bigint, f_in bigint, f_out bigint, pattern integer, subnet integer, subnetset integer, t timestamp without time zone) RETURNS void
    LANGUAGE plpgsql
    AS $_$
DECLARE
  row RECORD;
  tablename text;
  sel text;
  upd text;
BEGIN
  tablename := 'netflow.pattern_subnet_total_' || g || '_' || s;
  sel := 'SELECT byte_in,byte_out,pack_in,pack_out,flow_in,flow_out FROM ' || tablename::regclass || ' WHERE time = $1 AND pattern = $2 ';
  IF subnet IS NULL AND subnetset IS NULL THEN
   sel := sel || 'AND subnet IS NULL AND set IS NULL';
   EXECUTE sel INTO row USING t, pattern;
  ELSIF subnetset IS NULL THEN
   sel := sel || 'AND subnet = $3 AND set IS NULL';
   EXECUTE sel INTO row USING t, pattern, subnet;
  ELSIF subnet IS NULL THEN
   sel := sel || 'AND set = $3 AND subnet IS NULL';
   EXECUTE sel INTO row USING t, pattern, subnetset;
  END IF;
  
  IF row IS NOT NULL THEN
    upd := 'UPDATE ' || tablename::regclass || ' SET byte_in = $1, byte_out = $2, pack_in = $3, pack_out = $4, flow_in = $5, flow_out = $6 WHERE time = $7 AND pattern = $8 ';
	IF subnet IS NULL AND subnetset IS NULL THEN
	  upd := upd || 'AND subnet IS NULL AND set IS NULL';
	  EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, pattern;
    ELSIF subnetset IS NULL THEN
      upd := upd || 'AND subnet = $9 AND set IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, pattern, subnet;
    ELSIF subnet IS NULL THEN
      upd := upd || 'AND set = $9 AND subnet IS NULL';
      EXECUTE upd USING row.byte_in+b_in, row.byte_out+b_out, row.pack_in+p_in, row.pack_out+p_out, row.flow_in+f_in, row.flow_out+f_out, t, pattern, subnetset;
    END IF;
  ELSE 
    EXECUTE 'INSERT INTO ' || tablename::regclass || '(time, pattern, subnet, set, byte_in, byte_out, pack_in, pack_out, flow_in, flow_out) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)' USING t, pattern, subnet, subnetset, b_in, b_out, p_in, p_out, f_in, f_out;
  END IF;
END;
$_$;

SET search_path = core, pg_catalog;

SET default_tablespace = '';

SET default_with_oids = false;

CREATE TABLE cip_sys_address_type (
    id_cip_sys_address_type integer NOT NULL,
    name character varying(50) NOT NULL,
    ind_physical boolean DEFAULT false NOT NULL
);

COMMENT ON TABLE cip_sys_address_type IS 'Pokazuje vrstu adrese';

COMMENT ON COLUMN cip_sys_address_type.ind_physical IS 'da li je adresa na fizickom sloju (MAC)';

CREATE TABLE cip_sys_module (
    id_cip_sys_module integer NOT NULL,
    name character varying(50) NOT NULL,
    tag character varying(30) NOT NULL,
    description character varying(100),
    ordering integer NOT NULL,
    settings_item character varying(100)
);

COMMENT ON TABLE cip_sys_module IS 'vrste modula u NetVizura sistemu - IS, MIB, VIRC ...  TODO - Potreban je i modul ALL koji zaokruzuje sve module - pre svega zbog privilegija';

CREATE TABLE cip_usr_looking_glass_command (
    id_cip_usr_looking_glass_command integer NOT NULL,
    command character varying(250) DEFAULT NULL::character varying,
    description character varying(250) DEFAULT NULL::character varying,
    ind_has_arguments boolean DEFAULT false NOT NULL
);

COMMENT ON TABLE cip_usr_looking_glass_command IS 'spisak looking glass komandi koje se mogu pojaviti na device-ima';

CREATE SEQUENCE cip_usr_looking_glass_command_id_cip_usr_looking_glass_comm_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE cip_usr_looking_glass_command_id_cip_usr_looking_glass_comm_seq OWNED BY cip_usr_looking_glass_command.id_cip_usr_looking_glass_command;

CREATE TABLE criterion (
    id bigint NOT NULL,
    owner_id bigint,
    simple_field_name character varying,
    simple_param_1 character varying,
    simple_param_2 character varying,
    comparison_op character varying,
    composite_name character varying,
    logical_op character varying,
    dtype character varying(30)
);

CREATE SEQUENCE criterion_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE criterion_id_seq OWNED BY criterion.id;

CREATE TABLE favorites (
    id integer NOT NULL,
    id_user integer,
    module character varying(30),
    id_node integer,
    classtype character varying,
    id_ancestor_node integer,
    ancestor_classtype character varying,
    additional_properties character varying
);

CREATE SEQUENCE favorites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE favorites_id_seq OWNED BY favorites.id;

CREATE TABLE image (
    id_image integer NOT NULL,
    name character varying(250) NOT NULL,
    content bytea
);

COMMENT ON TABLE image IS 'interne slicice u bazi  TODO - kako ce ovo funkcionisati sa GWT-om - sa servletima nije moglo';

CREATE SEQUENCE image_id_image_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE image_id_image_seq OWNED BY image.id_image;

CREATE TABLE permission_module (
    id_permission_module integer NOT NULL,
    id_cip_sys_module integer NOT NULL,
    id_role integer NOT NULL,
    security_level character varying(30) NOT NULL
);

COMMENT ON TABLE permission_module IS 'pokazuje dozvolu za dati modul i datu rolu  index za id_node_role';

CREATE SEQUENCE permission_module_id_permission_module_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE permission_module_id_permission_module_seq OWNED BY permission_module.id_permission_module;

CREATE TABLE property (
    id_property integer NOT NULL,
    name character varying(250) NOT NULL,
    description character varying(250),
    unit character varying(100),
    dtype character varying(32),
    value_type character varying(32)
);

COMMENT ON TABLE property IS 'opsti property za nodove';

CREATE TABLE property_expression (
    id_property integer NOT NULL,
    expression character varying(250) NOT NULL
);

COMMENT ON TABLE property_expression IS 'property tipa aritmeticko - logickog izraza ';

CREATE SEQUENCE property_id_property_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE property_id_property_seq OWNED BY property.id_property;

CREATE TABLE property_regex (
    id_property integer NOT NULL,
    regex character varying(250) NOT NULL
);

COMMENT ON TABLE property_regex IS 'property tipa regularnog izraza';

CREATE TABLE property_simple (
    id_property integer NOT NULL,
    value text
);

CREATE TABLE property_snmp (
    id_property integer NOT NULL,
    id_mib_element integer NOT NULL
);

COMMENT ON TABLE property_snmp IS 'property tipa SNMP OID-a';

COMMENT ON COLUMN property_snmp.id_mib_element IS 'druga snmp baza // Zula';

SET search_path = device, pg_catalog;

CREATE TABLE device (
    id_device integer NOT NULL,
    name character varying(256),
    description text,
    ip_address inet
);

CREATE SEQUENCE device_id_device_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE device_id_device_seq OWNED BY device.id_device;

CREATE TABLE resource (
    id_resource integer NOT NULL,
    id_device integer,
    id_subdevice integer,
    id_resource_type integer,
    discriminator character varying(32),
    oid text,
    value text,
    date_last_change timestamp without time zone,
    name character varying(256),
    unit character varying(20)
);

CREATE SEQUENCE resource_id_resource_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE resource_id_resource_seq OWNED BY resource.id_resource;

CREATE TABLE subdevice (
    id_subdevice integer NOT NULL,
    id_device integer NOT NULL,
    id_subdevice_type integer NOT NULL,
    id_parent integer,
    name character varying(256),
    index text,
    description text
);

CREATE SEQUENCE subdevice_id_subdevice_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE subdevice_id_subdevice_seq OWNED BY subdevice.id_subdevice;

SET search_path = event_alarm_action, pg_catalog;

CREATE TABLE event (
    id_event integer NOT NULL,
    id_cip_sys_module integer NOT NULL,
    enum_module_event_category character(32),
    ind_notification boolean NOT NULL,
    fingerprint_ident bigint,
    activator_desc character varying(255),
    source_desc character varying(255),
    source_ip inet,
    time_start timestamp without time zone NOT NULL,
    time_end timestamp without time zone,
    message_start character varying(255) NOT NULL,
    message_end character varying(255),
    level integer NOT NULL
);

COMMENT ON COLUMN event.id_cip_sys_module IS 'modul iz koga dolazi event';

COMMENT ON COLUMN event.enum_module_event_category IS 'kategorija eventa u datom modulu ukoliko modul kreira raznorodne evente';

COMMENT ON COLUMN event.ind_notification IS 'indikator da li je event vrste NOTIFICATION_EVENT (koji samo okida) ili ALERT_EVENT (koji ima good i bad stanje). True - NOTIFICATION, False - Alert';

COMMENT ON COLUMN event.fingerprint_ident IS 'neka vrsta identifikatora activatora+source-a kako bi se brzo pronasli dogadjaji koji su:
1) ISTOG tipa (tj. definisani od strane istog alarma)
2) se dogodili na ISTOM mestu (tj. na istom senderu)';

COMMENT ON COLUMN event.activator_desc IS 'naziv uslova ili alarma koji je okinuo dogadjaj';

COMMENT ON COLUMN event.source_desc IS 'opis objekta na kom se dogadjaj dogodio - konverzacija (za mediju) ili monitor (za netis) ili syslog poruka (u slucaju sysloga) ili SNMP trap poruka (u slucaju trapa)... Može biti prazno ako nema smisla';

COMMENT ON COLUMN event.source_ip IS 'IP adresa odakle je potekao event. Eksporter (medija), device ili subdevice (netis), uredjaj (trap i syslog); može biti prazno ako nema smisla';

COMMENT ON COLUMN event.message_start IS 'poruka koju activator salje pri pocetnom okidanju eventa u slucaju tipa alert';

COMMENT ON COLUMN event.message_end IS 'poruka koju activator salje pri zavrsnom okidanju eventa u slucaju tipa alert';

CREATE TABLE event_hidden (
    id_event_hidden integer NOT NULL,
    id_user bigint NOT NULL,
    id_event bigint NOT NULL
);

CREATE SEQUENCE event_hidden_id_event_hidden_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE event_hidden_id_event_hidden_seq OWNED BY event_hidden.id_event_hidden;

CREATE SEQUENCE event_id_event_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE event_id_event_seq OWNED BY event.id_event;

CREATE TABLE event_resolution_type (
    id_resolution_type integer NOT NULL,
    type character varying(10) NOT NULL,
    description text
);

CREATE SEQUENCE event_resolution_type_id_resolution_type_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE event_resolution_type_id_resolution_type_seq OWNED BY event_resolution_type.id_resolution_type;

CREATE TABLE event_state (
    id_event_state integer NOT NULL,
    id_event bigint NOT NULL,
    state smallint NOT NULL,
    action_time timestamp without time zone NOT NULL,
    action_user bigint NOT NULL,
    comment text,
    resolution_type smallint,
    resolution_time timestamp without time zone
);

CREATE SEQUENCE event_state_id_event_state_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE event_state_id_event_state_seq OWNED BY event_state.id_event_state;

CREATE TABLE event_ticket_state (
    id_ticket integer NOT NULL,
    id_event bigint NOT NULL,
    action_time timestamp without time zone NOT NULL,
    action_by_user bigint NOT NULL,
    assigned_to_user bigint NOT NULL,
    ticket_state smallint NOT NULL,
    comment text,
    resolution_type smallint,
    resolution_time timestamp without time zone
);

CREATE SEQUENCE event_ticket_state_id_ticket_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE event_ticket_state_id_ticket_seq OWNED BY event_ticket_state.id_ticket;

SET search_path = filter, pg_catalog;

CREATE TABLE action (
    id integer NOT NULL,
    criterion_id integer,
    action character varying(30) NOT NULL,
    description character varying,
    owner_id integer NOT NULL,
    ordering integer NOT NULL,
    disabled boolean DEFAULT false,
    name character varying
);

CREATE TABLE action_set (
    id integer NOT NULL,
    service_name character varying NOT NULL,
    default_action character varying(30)
);

CREATE SEQUENCE action_set_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE action_set_id_seq OWNED BY action_set.id;

CREATE SEQUENCE actions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE actions_id_seq OWNED BY action.id;

SET search_path = flow, pg_catalog;

CREATE TABLE flow (
    id bigint NOT NULL,
    srcaddr inet,
    srcport integer,
    dstaddr inet,
    dstport integer,
    protocol integer,
    expaddr inet NOT NULL,
    flowstart bigint,
    flowend bigint,
    fields bytea
);

CREATE SEQUENCE flow_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE flow_id_seq OWNED BY flow.id;

CREATE TABLE media_alarm (
    id bigint NOT NULL,
    name character varying(100),
    message text,
    level integer,
    criterion bigint NOT NULL
);

CREATE SEQUENCE media_alarm_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE media_alarm_id_seq OWNED BY media_alarm.id;

CREATE TABLE media_alarm_stats (
    id bigint NOT NULL,
    convid bigint,
    level integer,
    count integer
);

CREATE SEQUENCE media_alarm_stats_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE media_alarm_stats_id_seq OWNED BY media_alarm_stats.id;

CREATE TABLE media_application (
    id bigint NOT NULL,
    name character varying,
    description character varying,
    criterion bigint,
    ordering integer
);

CREATE SEQUENCE media_application_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE media_application_id_seq OWNED BY media_application.id;

CREATE TABLE media_conversation (
    id bigint NOT NULL,
    srcaddr inet,
    srcport integer,
    dstaddr inet,
    dstport integer,
    protocol smallint,
    ssrc bigint,
    dscp smallint,
    minbytes bigint,
    maxbytes bigint,
    minmediabytes bigint,
    maxmediabytes bigint,
    minpackets bigint,
    maxpackets bigint,
    minmediapackets bigint,
    maxmediapackets bigint,
    minpacketloss bigint,
    maxpacketloss bigint,
    packetdrops bigint,
    minpacketexpected bigint,
    maxpacketexpected bigint,
    minlossevents bigint,
    maxlossevents bigint,
    minbitrate bigint,
    maxbitrate bigint,
    minmediabitrate bigint,
    maxmediabitrate bigint,
    minmediapacketrate bigint,
    maxmediapacketrate bigint,
    minmediaratevariation integer,
    maxmediaratevariation integer,
    minlossrate integer,
    maxlossrate integer,
    minjitter bigint,
    maxjitter bigint,
    minrtt bigint,
    maxrtt bigint,
    minttl smallint,
    maxttl smallint,
    starttime bigint,
    endtime bigint,
    maxmediaerrorcnt integer,
    active boolean,
    nbar character varying(30),
    application character varying(255),
    exporters smallint DEFAULT 1
);

CREATE SEQUENCE media_conversation_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE media_conversation_id_seq OWNED BY media_conversation.id;

CREATE TABLE media_event (
    id bigint NOT NULL,
    id_conversation bigint NOT NULL,
    id_event bigint NOT NULL
);

CREATE SEQUENCE media_event_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE media_event_id_seq OWNED BY media_event.id;

CREATE TABLE media_sample (
    id integer NOT NULL,
    id_conversation bigint NOT NULL,
    id_flow bigint NOT NULL
);

CREATE SEQUENCE media_sample_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE media_sample_id_seq OWNED BY media_sample.id;

SET search_path = framework, pg_catalog;

CREATE TABLE db_version (
	id serial NOT NULL,
	patch_number integer NOT NULL,
	patch_date timestamp without time zone NOT NULL
);

COMMENT ON TABLE framework.db_version IS 'Tabela koja prati revizije baze. Sadrzi trenutnu patch reviziju (ceo broj) kao i datum kada je ta patch revizija primenjena. Treba da postoji samo jedan unos sa vrednoscu ID-a 1';

CREATE TABLE module (
    id_module integer NOT NULL,
    name character varying(255) NOT NULL,
    tag character varying(30) NOT NULL,
    description character varying(255),
    "order" integer NOT NULL
);

CREATE SEQUENCE module_id_module_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE module_id_module_seq OWNED BY module.id_module;

SET search_path = login, pg_catalog;

CREATE TABLE principal (
    id_role integer NOT NULL,
    id_role_c integer,
    date_c timestamp without time zone,
    id_role_m integer,
    date_m timestamp without time zone,
    dtype character varying(32)
);

CREATE SEQUENCE role_id_role_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE role_id_role_seq OWNED BY principal.id_role;

CREATE TABLE "user" (
    id_role integer NOT NULL,
    username character varying(250) NOT NULL,
    password character varying(250) NOT NULL,
    first_name character varying(250) NOT NULL,
    last_name character varying(250) NOT NULL,
    ind_active boolean DEFAULT true NOT NULL,
    ind_administrator boolean DEFAULT false NOT NULL,
    ind_multiple_login boolean DEFAULT false NOT NULL,
    address character varying,
    email character varying,
    phone character varying,
    mobile character varying
);

CREATE TABLE usergroup (
    id_role integer NOT NULL,
    name character varying(250) NOT NULL
);

CREATE TABLE usergroup_user (
    id_usergroup_user integer NOT NULL,
    id_role_usergroup integer NOT NULL,
    id_role_user integer NOT NULL,
    id_role_c integer NOT NULL,
    date_c timestamp without time zone NOT NULL,
    id_role_d integer,
    date_d timestamp without time zone
);

CREATE SEQUENCE usergroup_user_id_usergroup_user_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE usergroup_user_id_usergroup_user_seq OWNED BY usergroup_user.id_usergroup_user;

SET search_path = mapping, pg_catalog;

CREATE TABLE asinfo (
    asnumber bigint NOT NULL,
    asname character varying(255) NOT NULL,
    ascolor character varying(6),
    description character varying(255)
);

CREATE TABLE protocol (
    protocolname character varying(255) NOT NULL,
    protocolnumber integer NOT NULL,
    protocoldescription character varying(255) NOT NULL,
    protocolcolor character varying(6)
);

CREATE TABLE qos_dscp (
    dscp integer NOT NULL,
    dscpname character varying(255),
    dscpdescription character varying(255),
    dscpcolor character varying(6)
);

CREATE TABLE qos_ipprecedence (
    ipprecedence integer NOT NULL,
    ipprecedencename character varying(255),
    ipprecedencecolor character varying(6)
);

CREATE TABLE qos_tos (
    tosdecimal integer NOT NULL,
    tosdescription character varying(255),
    ipprecedence integer,
    dscp integer,
    toscolor character varying(6)
);

CREATE TABLE service (
    id integer NOT NULL,
    servicename character varying(255) NOT NULL,
    servicecolor character varying(6),
    description character varying(255)
);

CREATE SEQUENCE service_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE service_id_seq OWNED BY service.id;

CREATE TABLE service_port (
    port integer NOT NULL,
    id_service integer NOT NULL
);

SET search_path = mib, pg_catalog;

CREATE TABLE element_module (
    element_id integer NOT NULL,
    module_id integer NOT NULL
);

CREATE TABLE mib_capability_support (
    id_capability_support integer NOT NULL,
    id_mib_element integer NOT NULL,
    name character varying(255) NOT NULL,
    includes text
);

CREATE SEQUENCE id_capability_support_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_capability_support_seq OWNED BY mib_capability_support.id_capability_support;

CREATE TABLE mib_capability_support_variation (
    id_capability_support_variation integer NOT NULL,
    id_capability_support integer NOT NULL,
    name character varying(255) NOT NULL,
    description text,
    access character varying(30) NOT NULL,
    syntax text,
    write_syntax text,
    creation_requirement text,
    default_value character varying(255)
);

CREATE SEQUENCE id_capability_support_variation_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_capability_support_variation_seq OWNED BY mib_capability_support_variation.id_capability_support_variation;

CREATE TABLE mib_compliance_module_group (
    id_compliance_module_group integer NOT NULL,
    id_compliance_module integer NOT NULL,
    compliance_group character varying(255) NOT NULL,
    description text
);

CREATE SEQUENCE id_compliance_module_group_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_compliance_module_group_seq OWNED BY mib_compliance_module_group.id_compliance_module_group;

CREATE TABLE mib_compliance_module_object (
    id_compliance_module_object integer NOT NULL,
    id_compliance_module integer NOT NULL,
    name character varying(255) NOT NULL,
    description text,
    minimum_access character varying(30) NOT NULL,
    syntax text,
    write_syntax text
);

CREATE SEQUENCE id_compliance_module_object_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_compliance_module_object_seq OWNED BY mib_compliance_module_object.id_compliance_module_object;

CREATE TABLE mib_compliance_module (
    id_compliance_module integer NOT NULL,
    id_mib_element integer NOT NULL,
    name character varying(255) NOT NULL,
    mandatory_groups text
);

CREATE SEQUENCE id_compliance_module_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_compliance_module_seq OWNED BY mib_compliance_module.id_compliance_module;

CREATE TABLE mib_identity_revision (
    id_identity_revision integer NOT NULL,
    id_mib_element integer NOT NULL,
    description text,
    revision timestamp without time zone
);

CREATE SEQUENCE id_identity_revision_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_identity_revision_seq OWNED BY mib_identity_revision.id_identity_revision;

CREATE TABLE mib_element (
    id_mib_element integer NOT NULL,
    oid text NOT NULL,
    name character varying(255) NOT NULL,
    id_mib_element_parent integer,
    status character varying(30) NOT NULL,
    reference character varying(255),
    description text,
    dtype character varying(32)
);

CREATE SEQUENCE id_mib_element_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_mib_element_seq OWNED BY mib_element.id_mib_element;

CREATE TABLE mib_module_import (
    id_mib_module_import integer NOT NULL,
    id_mib_module integer NOT NULL,
    import character varying(255) NOT NULL,
    id_mib_module_import_from integer NOT NULL
);

CREATE SEQUENCE id_mib_module_import_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_mib_module_import_seq OWNED BY mib_module_import.id_mib_module_import;

CREATE TABLE mib_module (
    id_mib_module integer NOT NULL,
    name character varying(255) NOT NULL,
    release_date timestamp without time zone
);

CREATE SEQUENCE id_mib_module_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_mib_module_seq OWNED BY mib_module.id_mib_module;

CREATE TABLE mib_textual_convention (
    id_mib_textual_convention integer NOT NULL,
    name character varying(255) NOT NULL,
    description text,
    display_hint text,
    syntax text NOT NULL,
    status character varying(30) NOT NULL,
    reference text,
    id_mib_module integer NOT NULL
);

CREATE SEQUENCE id_mib_textual_convention_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE id_mib_textual_convention_seq OWNED BY mib_textual_convention.id_mib_textual_convention;

CREATE TABLE mib_app_property (
    id_mib_app_property character varying(255) NOT NULL,
    value character varying(255) NOT NULL
);

CREATE TABLE mib_capability (
    id_mib_element integer NOT NULL,
    product_release character varying(255)
);

CREATE TABLE mib_compliance (
    id_mib_element integer NOT NULL
);

CREATE TABLE mib_favorite (
    id bigint NOT NULL,
    id_element bigint,
    id_user bigint
);

CREATE SEQUENCE mib_favorite_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE mib_favorite_id_seq OWNED BY mib_favorite.id;

CREATE TABLE mib_identity (
    id_mib_element integer NOT NULL,
    organization text NOT NULL,
    contact_info text NOT NULL,
    last_updated timestamp without time zone
);

CREATE TABLE mib_notification_group (
    id_mib_element integer NOT NULL,
    notifications text
);

CREATE TABLE mib_notification_type (
    id_mib_element integer NOT NULL,
    objects text
);

CREATE TABLE mib_object_group (
    id_mib_element integer NOT NULL,
    objects text
);

CREATE TABLE mib_object_type (
    id_mib_element integer NOT NULL,
    syntax text NOT NULL,
    syntax_values text,
    units character varying(255),
    access character varying(30) NOT NULL,
    index text,
    default_value character varying(255)
);

SET search_path = multilog, pg_catalog;

CREATE TABLE severity_exporter_counter (
    id integer NOT NULL,
    "time" timestamp without time zone,
    severity integer,
    exporter inet,
    count bigint
);

CREATE SEQUENCE exporter_severity_counter_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_severity_counter_id_seq OWNED BY severity_exporter_counter.id;

CREATE TABLE performance (
    id integer NOT NULL,
    "time" timestamp without time zone,
    taken bigint,
    dropped bigint,
    obfuscated bigint
);

CREATE SEQUENCE performance_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE performance_id_seq OWNED BY performance.id;

CREATE TABLE syslog_message (
    id integer NOT NULL,
    "time" timestamp without time zone,
    exporter inet,
    severity integer,
    facility integer,
    message text
);

CREATE SEQUENCE syslog_message_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE syslog_message_id_seq OWNED BY syslog_message.id;

CREATE TABLE trap_message (
    id integer NOT NULL,
    "time" timestamp without time zone,
    exporter inet,
    trap_oid text,
    message text,
    request_id integer,
    error_status integer,
    error_index integer,
    sysup_time bigint
);

CREATE SEQUENCE trap_message_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE trap_message_id_seq OWNED BY trap_message.id;

CREATE TABLE trap_variables (
    oid text,
    value text,
    owner_id integer NOT NULL,
    id integer NOT NULL
);

CREATE SEQUENCE trap_variables_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE trap_variables_id_seq OWNED BY trap_variables.id;

CREATE SEQUENCE trap_variables_owner_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE trap_variables_owner_id_seq OWNED BY trap_variables.owner_id;

SET search_path = netflow, pg_catalog;

CREATE TABLE alarm (
    id bigint NOT NULL,
    name character varying(100),
    message text,
    level integer,
    scopetype character varying(20),
    scopeid character varying(100),
    objecttype character varying(20),
    objectid character varying(100),
    criterion bigint NOT NULL,
    email character varying(255)
);

CREATE SEQUENCE alarm_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE alarm_id_seq OWNED BY alarm.id;

CREATE TABLE counter_config (
    id bigint NOT NULL,
    type character varying(20),
    exporter inet,
    interface integer,
    pattern bigint,
    subnet bigint,
    value text,
    name character varying(100),
    subnet_set bigint
);

CREATE SEQUENCE counter_config_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE counter_config_id_seq OWNED BY counter_config.id;

CREATE TABLE default_config (
    id integer NOT NULL,
    type character varying(20) NOT NULL,
    top_as integer,
    top_conversation integer,
    top_host integer,
    top_protocol integer,
    top_qos integer,
    top_service integer,
    top_children integer
);

CREATE SEQUENCE default_config_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE default_config_id_seq OWNED BY default_config.id;

CREATE TABLE exporter_device_as_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    "as" integer NOT NULL,
    byte_src bigint,
    byte_dst bigint,
    pack_src bigint,
    pack_dst bigint,
    flow_src bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_as_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_as_g1_id_seq OWNED BY exporter_device_as_g1.id;

CREATE TABLE exporter_device_as_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    "as" integer NOT NULL,
    byte_src bigint,
    byte_dst bigint,
    pack_src bigint,
    pack_dst bigint,
    flow_src bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_as_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_as_g2_id_seq OWNED BY exporter_device_as_g2.id;

CREATE TABLE exporter_device_as_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    "as" integer NOT NULL,
    byte_src bigint,
    byte_dst bigint,
    pack_src bigint,
    pack_dst bigint,
    flow_src bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_as_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_as_g3_id_seq OWNED BY exporter_device_as_g3.id;

CREATE TABLE exporter_device_conversation_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_conversation_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_conversation_g1_id_seq OWNED BY exporter_device_conversation_g1.id;

CREATE TABLE exporter_device_conversation_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_conversation_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_conversation_g2_id_seq OWNED BY exporter_device_conversation_g2.id;

CREATE TABLE exporter_device_conversation_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_conversation_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_conversation_g3_id_seq OWNED BY exporter_device_conversation_g3.id;

CREATE TABLE exporter_device_host_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    host inet NOT NULL,
    byte_src bigint,
    byte_dst bigint,
    pack_src bigint,
    pack_dst bigint,
    flow_src bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_host_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_host_g1_id_seq OWNED BY exporter_device_host_g1.id;

CREATE TABLE exporter_device_host_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    host inet NOT NULL,
    byte_src bigint,
    byte_dst bigint,
    pack_src bigint,
    pack_dst bigint,
    flow_src bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_host_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_host_g2_id_seq OWNED BY exporter_device_host_g2.id;

CREATE TABLE exporter_device_host_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    host inet NOT NULL,
    byte_src bigint,
    byte_dst bigint,
    pack_src bigint,
    pack_dst bigint,
    flow_src bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_host_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_host_g3_id_seq OWNED BY exporter_device_host_g3.id;

CREATE TABLE exporter_device_protocol_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    protocol integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_protocol_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_protocol_g1_id_seq OWNED BY exporter_device_protocol_g1.id;

CREATE TABLE exporter_device_protocol_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    protocol integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_protocol_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_protocol_g2_id_seq OWNED BY exporter_device_protocol_g2.id;

CREATE TABLE exporter_device_protocol_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    protocol integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_protocol_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_protocol_g3_id_seq OWNED BY exporter_device_protocol_g3.id;

CREATE TABLE exporter_device_qos_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    tos integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_qos_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_qos_g1_id_seq OWNED BY exporter_device_qos_g1.id;

CREATE TABLE exporter_device_qos_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    tos integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_qos_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_qos_g2_id_seq OWNED BY exporter_device_qos_g2.id;

CREATE TABLE exporter_device_qos_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    tos integer NOT NULL,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_qos_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_qos_g3_id_seq OWNED BY exporter_device_qos_g3.id;

CREATE TABLE exporter_device_service_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    port integer NOT NULL,
    byte_src bigint,
    pack_src bigint,
    flow_src bigint,
    byte_dst bigint,
    pack_dst bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_service_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_service_g1_id_seq OWNED BY exporter_device_service_g1.id;

CREATE TABLE exporter_device_service_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    port integer NOT NULL,
    byte_src bigint,
    pack_src bigint,
    flow_src bigint,
    byte_dst bigint,
    pack_dst bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_service_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_service_g2_id_seq OWNED BY exporter_device_service_g2.id;

CREATE TABLE exporter_device_service_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    port integer NOT NULL,
    byte_src bigint,
    pack_src bigint,
    flow_src bigint,
    byte_dst bigint,
    pack_dst bigint,
    flow_dst bigint
);

CREATE SEQUENCE exporter_device_service_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_service_g3_id_seq OWNED BY exporter_device_service_g3.id;

CREATE TABLE exporter_device_total_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_total_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_total_g1_id_seq OWNED BY exporter_device_total_g1.id;

CREATE TABLE exporter_device_total_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_total_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_total_g2_id_seq OWNED BY exporter_device_total_g2.id;

CREATE TABLE exporter_device_total_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet,
    byte bigint,
    pack bigint,
    flow bigint
);

CREATE SEQUENCE exporter_device_total_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_device_total_g3_id_seq OWNED BY exporter_device_total_g3.id;

CREATE TABLE exporter_interface_as_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    "as" integer NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_as_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_as_g1_id_seq OWNED BY exporter_interface_as_g1.id;

CREATE TABLE exporter_interface_as_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    "as" integer NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_as_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_as_g2_id_seq OWNED BY exporter_interface_as_g2.id;

CREATE TABLE exporter_interface_as_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    "as" integer NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_as_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_as_g3_id_seq OWNED BY exporter_interface_as_g3.id;

CREATE TABLE exporter_interface_conversation_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_conversation_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_conversation_g1_id_seq OWNED BY exporter_interface_conversation_g1.id;

CREATE TABLE exporter_interface_conversation_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_conversation_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_conversation_g2_id_seq OWNED BY exporter_interface_conversation_g2.id;

CREATE TABLE exporter_interface_conversation_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_conversation_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_conversation_g3_id_seq OWNED BY exporter_interface_conversation_g3.id;

CREATE TABLE exporter_interface_host_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    host inet NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_host_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_host_g1_id_seq OWNED BY exporter_interface_host_g1.id;

CREATE TABLE exporter_interface_host_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    host inet NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_host_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_host_g2_id_seq OWNED BY exporter_interface_host_g2.id;

CREATE TABLE exporter_interface_host_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    host inet NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_host_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_host_g3_id_seq OWNED BY exporter_interface_host_g3.id;

CREATE TABLE exporter_interface_protocol_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    protocol integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_protocol_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_protocol_g1_id_seq OWNED BY exporter_interface_protocol_g1.id;

CREATE TABLE exporter_interface_protocol_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    protocol integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_protocol_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_protocol_g2_id_seq OWNED BY exporter_interface_protocol_g2.id;

CREATE TABLE exporter_interface_protocol_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    protocol integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_protocol_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_protocol_g3_id_seq OWNED BY exporter_interface_protocol_g3.id;

CREATE TABLE exporter_interface_qos_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    tos integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_qos_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_qos_g1_id_seq OWNED BY exporter_interface_qos_g1.id;

CREATE TABLE exporter_interface_qos_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    tos integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_qos_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_qos_g2_id_seq OWNED BY exporter_interface_qos_g2.id;

CREATE TABLE exporter_interface_qos_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    tos integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_qos_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_qos_g3_id_seq OWNED BY exporter_interface_qos_g3.id;

CREATE TABLE exporter_interface_service_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    port integer NOT NULL,
    byte_in_src bigint,
    byte_out_src bigint,
    pack_in_src bigint,
    pack_out_src bigint,
    flow_in_src bigint,
    flow_out_src bigint,
    byte_in_dst bigint,
    byte_out_dst bigint,
    pack_in_dst bigint,
    pack_out_dst bigint,
    flow_in_dst bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_service_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_service_g1_id_seq OWNED BY exporter_interface_service_g1.id;

CREATE TABLE exporter_interface_service_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    port integer NOT NULL,
    byte_in_src bigint,
    byte_out_src bigint,
    pack_in_src bigint,
    pack_out_src bigint,
    flow_in_src bigint,
    flow_out_src bigint,
    byte_in_dst bigint,
    byte_out_dst bigint,
    pack_in_dst bigint,
    pack_out_dst bigint,
    flow_in_dst bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_service_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_service_g2_id_seq OWNED BY exporter_interface_service_g2.id;

CREATE TABLE exporter_interface_service_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    port integer NOT NULL,
    byte_in_src bigint,
    byte_out_src bigint,
    pack_in_src bigint,
    pack_out_src bigint,
    flow_in_src bigint,
    flow_out_src bigint,
    byte_in_dst bigint,
    byte_out_dst bigint,
    pack_in_dst bigint,
    pack_out_dst bigint,
    flow_in_dst bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE exporter_interface_service_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_service_g3_id_seq OWNED BY exporter_interface_service_g3.id;

CREATE TABLE exporter_interface_total_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_total_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_total_g1_id_seq OWNED BY exporter_interface_total_g1.id;

CREATE TABLE exporter_interface_total_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_total_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_total_g2_id_seq OWNED BY exporter_interface_total_g2.id;

CREATE TABLE exporter_interface_total_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    exporter inet NOT NULL,
    interface integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE exporter_interface_total_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE exporter_interface_total_g3_id_seq OWNED BY exporter_interface_total_g3.id;

CREATE TABLE pattern (
    id integer NOT NULL,
    name character varying(100) NOT NULL,
    description text
);

CREATE TABLE pattern_criteria (
    id integer NOT NULL,
    pattern_id integer NOT NULL,
    type character varying(20) NOT NULL,
    value text
);

CREATE SEQUENCE pattern_criteria_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_criteria_id_seq OWNED BY pattern_criteria.id;

CREATE SEQUENCE pattern_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_id_seq OWNED BY pattern.id;

CREATE TABLE pattern_subnet_as_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    "as" integer NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_as_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_as_g1_id_seq OWNED BY pattern_subnet_as_g1.id;

CREATE TABLE pattern_subnet_as_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    "as" integer NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_as_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_as_g2_id_seq OWNED BY pattern_subnet_as_g2.id;

CREATE TABLE pattern_subnet_as_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    "as" integer NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_as_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_as_g3_id_seq OWNED BY pattern_subnet_as_g3.id;

CREATE TABLE pattern_subnet_conversation_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_conversation_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_conversation_g1_id_seq OWNED BY pattern_subnet_conversation_g1.id;

CREATE TABLE pattern_subnet_conversation_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_conversation_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_conversation_g2_id_seq OWNED BY pattern_subnet_conversation_g2.id;

CREATE TABLE pattern_subnet_conversation_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    source inet NOT NULL,
    destination inet NOT NULL,
    service integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_conversation_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_conversation_g3_id_seq OWNED BY pattern_subnet_conversation_g3.id;

CREATE TABLE pattern_subnet_host_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    host inet NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_host_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_host_g1_id_seq OWNED BY pattern_subnet_host_g1.id;

CREATE TABLE pattern_subnet_host_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    host inet NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_host_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_host_g2_id_seq OWNED BY pattern_subnet_host_g2.id;

CREATE TABLE pattern_subnet_host_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    host inet NOT NULL,
    byte_in_src bigint,
    byte_in_dst bigint,
    byte_out_src bigint,
    byte_out_dst bigint,
    pack_in_src bigint,
    pack_in_dst bigint,
    pack_out_src bigint,
    pack_out_dst bigint,
    flow_in_src bigint,
    flow_in_dst bigint,
    flow_out_src bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_host_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_host_g3_id_seq OWNED BY pattern_subnet_host_g3.id;

CREATE TABLE pattern_subnet_protocol_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    protocol integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_protocol_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_protocol_g1_id_seq OWNED BY pattern_subnet_protocol_g1.id;

CREATE TABLE pattern_subnet_protocol_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    protocol integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_protocol_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_protocol_g2_id_seq OWNED BY pattern_subnet_protocol_g2.id;

CREATE TABLE pattern_subnet_protocol_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    protocol integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_protocol_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_protocol_g3_id_seq OWNED BY pattern_subnet_protocol_g3.id;

CREATE TABLE pattern_subnet_qos_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    tos integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_qos_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_qos_g1_id_seq OWNED BY pattern_subnet_qos_g1.id;

CREATE TABLE pattern_subnet_qos_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    tos integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_qos_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_qos_g2_id_seq OWNED BY pattern_subnet_qos_g2.id;

CREATE TABLE pattern_subnet_qos_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    tos integer NOT NULL,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_qos_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_qos_g3_id_seq OWNED BY pattern_subnet_qos_g3.id;

CREATE TABLE pattern_subnet_service_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    port integer NOT NULL,
    byte_in_src bigint,
    byte_out_src bigint,
    pack_in_src bigint,
    pack_out_src bigint,
    flow_in_src bigint,
    flow_out_src bigint,
    byte_in_dst bigint,
    byte_out_dst bigint,
    pack_in_dst bigint,
    pack_out_dst bigint,
    flow_in_dst bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_service_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_service_g1_id_seq OWNED BY pattern_subnet_service_g1.id;

CREATE TABLE pattern_subnet_service_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    port integer NOT NULL,
    byte_in_src bigint,
    byte_out_src bigint,
    pack_in_src bigint,
    pack_out_src bigint,
    flow_in_src bigint,
    flow_out_src bigint,
    byte_in_dst bigint,
    byte_out_dst bigint,
    pack_in_dst bigint,
    pack_out_dst bigint,
    flow_in_dst bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_service_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_service_g2_id_seq OWNED BY pattern_subnet_service_g2.id;

CREATE TABLE pattern_subnet_service_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    port integer NOT NULL,
    byte_in_src bigint,
    byte_out_src bigint,
    pack_in_src bigint,
    pack_out_src bigint,
    flow_in_src bigint,
    flow_out_src bigint,
    byte_in_dst bigint,
    byte_out_dst bigint,
    pack_in_dst bigint,
    pack_out_dst bigint,
    flow_in_dst bigint,
    flow_out_dst bigint
);

CREATE SEQUENCE pattern_subnet_service_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_service_g3_id_seq OWNED BY pattern_subnet_service_g3.id;

CREATE TABLE pattern_subnet_total_g1 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_total_g1_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_total_g1_id_seq OWNED BY pattern_subnet_total_g1.id;

CREATE TABLE pattern_subnet_total_g2 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_total_g2_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_total_g2_id_seq OWNED BY pattern_subnet_total_g2.id;

CREATE TABLE pattern_subnet_total_g3 (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    pattern integer,
    subnet integer,
    set integer,
    byte_in bigint,
    byte_out bigint,
    pack_in bigint,
    pack_out bigint,
    flow_in bigint,
    flow_out bigint
);

CREATE SEQUENCE pattern_subnet_total_g3_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE pattern_subnet_total_g3_id_seq OWNED BY pattern_subnet_total_g3.id;

CREATE TABLE performance (
    id bigint NOT NULL,
    "time" timestamp without time zone NOT NULL,
    packets bigint,
    stored bigint,
    missed bigint,
    taken bigint,
    dropped bigint,
    counted bigint,
    write bigint,
    heap bigint,
    discards bigint
);

CREATE SEQUENCE performance_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE performance_id_seq OWNED BY performance.id;

CREATE TABLE set_in_pattern (
    id integer NOT NULL,
    id_pattern bigint NOT NULL,
    id_set bigint NOT NULL
);

CREATE SEQUENCE set_in_pattern_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE set_in_pattern_id_seq OWNED BY set_in_pattern.id;

CREATE TABLE subnet (
    id integer NOT NULL,
    address inet NOT NULL,
    name character varying(100) NOT NULL,
    description text,
    id_parent integer,
    id_set integer
);

CREATE SEQUENCE subnet_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE subnet_id_seq OWNED BY subnet.id;

CREATE TABLE subnet_in_pattern (
    id integer NOT NULL,
    id_pattern bigint NOT NULL,
    id_subnet bigint NOT NULL
);

CREATE SEQUENCE subnet_in_pattern_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE subnet_in_pattern_id_seq OWNED BY subnet_in_pattern.id;

CREATE TABLE subnet_set (
    id integer NOT NULL,
    name character varying(100) NOT NULL,
    description text NOT NULL,
    id_parent integer
);

CREATE SEQUENCE subnet_set_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE subnet_set_id_seq OWNED BY subnet_set.id;

SET search_path = policy, pg_catalog;

CREATE TABLE device_policy_console (
    id_device integer NOT NULL,
    id_policy_console integer NOT NULL
);

CREATE TABLE device_policy_snmp (
    id_device integer NOT NULL,
    id_policy_snmp integer NOT NULL
);

CREATE TABLE policy_console (
    id_policy_console integer NOT NULL,
    name character varying(100),
    ssh_key character varying(255),
    username character varying(100) NOT NULL,
    password character varying(100) NOT NULL,
    console_protocol character varying(32) NOT NULL
);

COMMENT ON COLUMN policy_console.ssh_key IS 'private KEY koji ce biti razmenjen sa serverom koji podrzava asimetricnu razmenu kljuceva';

CREATE SEQUENCE policy_console_id_policy_console_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE policy_console_id_policy_console_seq OWNED BY policy_console.id_policy_console;

CREATE TABLE policy_snmp (
    id_policy_snmp integer NOT NULL,
    name character varying(100),
    port integer DEFAULT 161 NOT NULL,
    retries integer DEFAULT 0 NOT NULL,
    timeout integer DEFAULT 1500 NOT NULL,
    max_bulk_repetitions integer DEFAULT 0 NOT NULL,
    community character varying(100),
    snmp_version character varying(32) NOT NULL,
    snmpv3_username character varying(100),
    snmpv3_auth_password character varying(100),
    snmpv3_priv_password character varying(100),
    snmpv3_security_level character varying(32),
    snmpv3_authentication_protocol character varying(32),
    snmpv3_privacy_protocol character varying(32),
    privileges character varying(16)
);

COMMENT ON COLUMN policy_snmp.max_bulk_repetitions IS 'koliko treba da bude veliki zahtev BULK tipa';

COMMENT ON COLUMN policy_snmp.snmpv3_auth_password IS 'najmanje 8 karaktera';

COMMENT ON COLUMN policy_snmp.snmpv3_priv_password IS 'najmanje 8 karaktera';

CREATE SEQUENCE policy_snmp_id_policy_snmp_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE policy_snmp_id_policy_snmp_seq OWNED BY policy_snmp.id_policy_snmp;

SET search_path = public, pg_catalog;

CREATE TABLE system_property (
    name character varying(100) NOT NULL,
    tag character varying(50) NOT NULL,
    type character varying(32) NOT NULL,
    value character varying(255)
);

CREATE TABLE user_property (
    name character varying(100) NOT NULL,
    tag character varying(50) NOT NULL,
    username character varying(50) NOT NULL,
    type character varying(32) NOT NULL,
    value character varying(255)
);

SET search_path = report, pg_catalog;

CREATE TABLE report (
    id integer NOT NULL,
    name character varying NOT NULL
);

CREATE SEQUENCE report_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_id_seq OWNED BY report.id;

CREATE TABLE report_query_from (
    id integer NOT NULL,
    id_mib_element integer,
    id_section integer NOT NULL
);

CREATE SEQUENCE report_query_from_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_query_from_id_seq OWNED BY report_query_from.id;

CREATE TABLE report_query_join (
    id integer NOT NULL,
    id_section integer NOT NULL
);

CREATE TABLE report_query_join_chunk (
    id integer NOT NULL,
    id_mib_element integer,
    side character varying(10) NOT NULL,
    id_join integer NOT NULL
);

CREATE SEQUENCE report_query_join_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_query_join_id_seq OWNED BY report_query_join_chunk.id;

CREATE SEQUENCE report_query_join_id_seq1
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_query_join_id_seq1 OWNED BY report_query_join.id;

CREATE TABLE report_query_selects (
    id integer NOT NULL,
    id_parent integer,
    operator character varying(5),
    type character varying(25) NOT NULL,
    id_mib_element integer,
    scalar_value character varying,
    id_mib_table integer
);

COMMENT ON COLUMN report_query_selects.id_parent IS 'Parent SelectQueryElement';

CREATE SEQUENCE report_query_selects_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_query_selects_id_seq OWNED BY report_query_selects.id;

CREATE TABLE report_section (
    id integer NOT NULL,
    id_report integer NOT NULL,
    name character varying NOT NULL,
    ordering integer NOT NULL,
    type character varying NOT NULL
);

CREATE SEQUENCE report_section_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_section_id_seq OWNED BY report_section.id;

CREATE TABLE report_table_section (
    id integer NOT NULL,
    table_description character varying NOT NULL
);

CREATE TABLE report_table_section_column (
    id integer NOT NULL,
    id_report_table_section integer NOT NULL,
    id_report_query_select integer,
    name character varying NOT NULL,
    id_mib_element integer,
    orderno integer DEFAULT 0 NOT NULL,
    type character varying(15)
);

CREATE SEQUENCE report_table_section_column_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE report_table_section_column_id_seq OWNED BY report_table_section_column.id;

SET search_path = typology, pg_catalog;

CREATE TABLE device_device_type (
    id_device integer NOT NULL,
    id_device_type integer NOT NULL
);

CREATE TABLE subdevice_type (
    id_subdevice_type integer NOT NULL,
    id_device_type integer,
    id_subdevice_category integer NOT NULL,
    id_parent integer,
    oid_enumerator text,
    ordering integer,
    ind_system boolean DEFAULT false,
    ind_active boolean DEFAULT true
);

CREATE SEQUENCE subdevice_type_id_subdevice_type_seq
    START WITH 10000
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE subdevice_type_id_subdevice_type_seq OWNED BY subdevice_type.id_subdevice_type;

CREATE TABLE resource_type (
    id_resource_type integer NOT NULL,
    id_device_type integer,
    id_subdevice_type integer,
    id_resource_category integer NOT NULL,
    discriminator character varying(32),
    name character varying(256),
    oid_prefix text,
    ind_name boolean DEFAULT false,
    ind_description boolean DEFAULT false,
    id_value_type integer,
    ordering integer,
    ind_system boolean DEFAULT false,
    ind_active boolean DEFAULT true
);

CREATE SEQUENCE resource_type_id_resource_type_seq
    START WITH 10000
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE resource_type_id_resource_type_seq OWNED BY resource_type.id_resource_type;

CREATE TABLE device_type (
    id_device_type integer NOT NULL,
    id_parent integer,
    name character varying(256),
    description text,
    ind_system boolean DEFAULT false,
    ind_active boolean DEFAULT true,
    id_test integer
);

CREATE SEQUENCE device_type_id_device_type_seq
    START WITH 10000
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE device_type_id_device_type_seq OWNED BY device_type.id_device_type;

CREATE TABLE resource_category (
    id_resource_category integer NOT NULL,
    id_subdevice_category integer,
    name character varying(256),
    description text,
    unit character varying(20),
    ind_system boolean DEFAULT false
);

CREATE SEQUENCE resource_category_id_resource_category_seq
    START WITH 10000
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE resource_category_id_resource_category_seq OWNED BY resource_category.id_resource_category;

CREATE TABLE subdevice_category (
    id_subdevice_category integer NOT NULL,
    name character varying(256),
    description character varying(512),
    ind_system boolean DEFAULT false
);

CREATE SEQUENCE subdevice_category_id_subdevice_category_seq
    START WITH 10000
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE subdevice_category_id_subdevice_category_seq OWNED BY subdevice_category.id_subdevice_category;

CREATE TABLE value_type (
    id_value_type integer NOT NULL,
    name character varying(256)
);

CREATE SEQUENCE value_type_id_value_type_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE value_type_id_value_type_seq OWNED BY value_type.id_value_type;

SET search_path = view, pg_catalog;

CREATE TABLE bookmark_item (
    id_bookmark_item integer NOT NULL,
    id_role integer,
    tag character varying(100) NOT NULL,
    objectid character varying(255) NOT NULL
);

COMMENT ON TABLE bookmark_item IS 'bookmark-ovi za korisnike za razne tipove drveta';

COMMENT ON COLUMN bookmark_item.objectid IS 'promisliti o ovakvom prezentovanju objekata';

CREATE SEQUENCE bookmark_item_id_bookmark_item_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE bookmark_item_id_bookmark_item_seq OWNED BY bookmark_item.id_bookmark_item;

SET search_path = core, pg_catalog;

ALTER TABLE ONLY cip_usr_looking_glass_command ALTER COLUMN id_cip_usr_looking_glass_command SET DEFAULT nextval('cip_usr_looking_glass_command_id_cip_usr_looking_glass_comm_seq'::regclass);

ALTER TABLE ONLY criterion ALTER COLUMN id SET DEFAULT nextval('criterion_id_seq'::regclass);

ALTER TABLE ONLY favorites ALTER COLUMN id SET DEFAULT nextval('favorites_id_seq'::regclass);

ALTER TABLE ONLY image ALTER COLUMN id_image SET DEFAULT nextval('image_id_image_seq'::regclass);

ALTER TABLE ONLY permission_module ALTER COLUMN id_permission_module SET DEFAULT nextval('permission_module_id_permission_module_seq'::regclass);

ALTER TABLE ONLY property ALTER COLUMN id_property SET DEFAULT nextval('property_id_property_seq'::regclass);

SET search_path = device, pg_catalog;

ALTER TABLE ONLY device ALTER COLUMN id_device SET DEFAULT nextval('device_id_device_seq'::regclass);

ALTER TABLE ONLY resource ALTER COLUMN id_resource SET DEFAULT nextval('resource_id_resource_seq'::regclass);

ALTER TABLE ONLY subdevice ALTER COLUMN id_subdevice SET DEFAULT nextval('subdevice_id_subdevice_seq'::regclass);

SET search_path = event_alarm_action, pg_catalog;

ALTER TABLE ONLY event ALTER COLUMN id_event SET DEFAULT nextval('event_id_event_seq'::regclass);

ALTER TABLE ONLY event_hidden ALTER COLUMN id_event_hidden SET DEFAULT nextval('event_hidden_id_event_hidden_seq'::regclass);

ALTER TABLE ONLY event_resolution_type ALTER COLUMN id_resolution_type SET DEFAULT nextval('event_resolution_type_id_resolution_type_seq'::regclass);

ALTER TABLE ONLY event_state ALTER COLUMN id_event_state SET DEFAULT nextval('event_state_id_event_state_seq'::regclass);

ALTER TABLE ONLY event_ticket_state ALTER COLUMN id_ticket SET DEFAULT nextval('event_ticket_state_id_ticket_seq'::regclass);

SET search_path = filter, pg_catalog;

ALTER TABLE ONLY action ALTER COLUMN id SET DEFAULT nextval('actions_id_seq'::regclass);

ALTER TABLE ONLY action_set ALTER COLUMN id SET DEFAULT nextval('action_set_id_seq'::regclass);

SET search_path = flow, pg_catalog;

ALTER TABLE ONLY flow ALTER COLUMN id SET DEFAULT nextval('flow_id_seq'::regclass);

ALTER TABLE ONLY media_alarm ALTER COLUMN id SET DEFAULT nextval('media_alarm_id_seq'::regclass);

ALTER TABLE ONLY media_alarm_stats ALTER COLUMN id SET DEFAULT nextval('media_alarm_stats_id_seq'::regclass);

ALTER TABLE ONLY media_application ALTER COLUMN id SET DEFAULT nextval('media_application_id_seq'::regclass);

ALTER TABLE ONLY media_conversation ALTER COLUMN id SET DEFAULT nextval('media_conversation_id_seq'::regclass);

ALTER TABLE ONLY media_event ALTER COLUMN id SET DEFAULT nextval('media_event_id_seq'::regclass);

ALTER TABLE ONLY media_sample ALTER COLUMN id SET DEFAULT nextval('media_sample_id_seq'::regclass);

SET search_path = framework, pg_catalog;

ALTER TABLE ONLY module ALTER COLUMN id_module SET DEFAULT nextval('module_id_module_seq'::regclass);

SET search_path = login, pg_catalog;

ALTER TABLE ONLY principal ALTER COLUMN id_role SET DEFAULT nextval('role_id_role_seq'::regclass);

ALTER TABLE ONLY usergroup_user ALTER COLUMN id_usergroup_user SET DEFAULT nextval('usergroup_user_id_usergroup_user_seq'::regclass);

SET search_path = mapping, pg_catalog;

ALTER TABLE ONLY service ALTER COLUMN id SET DEFAULT nextval('service_id_seq'::regclass);

SET search_path = mib, pg_catalog;

ALTER TABLE ONLY mib_capability_support ALTER COLUMN id_capability_support SET DEFAULT nextval('id_capability_support_seq'::regclass);

ALTER TABLE ONLY mib_capability_support_variation ALTER COLUMN id_capability_support_variation SET DEFAULT nextval('id_capability_support_variation_seq'::regclass);

ALTER TABLE ONLY mib_compliance_module ALTER COLUMN id_compliance_module SET DEFAULT nextval('id_compliance_module_seq'::regclass);

ALTER TABLE ONLY mib_compliance_module_group ALTER COLUMN id_compliance_module_group SET DEFAULT nextval('id_compliance_module_group_seq'::regclass);

ALTER TABLE ONLY mib_compliance_module_object ALTER COLUMN id_compliance_module_object SET DEFAULT nextval('id_compliance_module_object_seq'::regclass);

ALTER TABLE ONLY mib_element ALTER COLUMN id_mib_element SET DEFAULT nextval('id_mib_element_seq'::regclass);

ALTER TABLE ONLY mib_favorite ALTER COLUMN id SET DEFAULT nextval('mib_favorite_id_seq'::regclass);

ALTER TABLE ONLY mib_identity_revision ALTER COLUMN id_identity_revision SET DEFAULT nextval('id_identity_revision_seq'::regclass);

ALTER TABLE ONLY mib_module ALTER COLUMN id_mib_module SET DEFAULT nextval('id_mib_module_seq'::regclass);

ALTER TABLE ONLY mib_module_import ALTER COLUMN id_mib_module_import SET DEFAULT nextval('id_mib_module_import_seq'::regclass);

ALTER TABLE ONLY mib_textual_convention ALTER COLUMN id_mib_textual_convention SET DEFAULT nextval('id_mib_textual_convention_seq'::regclass);

SET search_path = multilog, pg_catalog;

ALTER TABLE ONLY performance ALTER COLUMN id SET DEFAULT nextval('performance_id_seq'::regclass);

ALTER TABLE ONLY severity_exporter_counter ALTER COLUMN id SET DEFAULT nextval('exporter_severity_counter_id_seq'::regclass);

ALTER TABLE ONLY syslog_message ALTER COLUMN id SET DEFAULT nextval('syslog_message_id_seq'::regclass);

ALTER TABLE ONLY trap_message ALTER COLUMN id SET DEFAULT nextval('trap_message_id_seq'::regclass);

ALTER TABLE ONLY trap_variables ALTER COLUMN owner_id SET DEFAULT nextval('trap_variables_owner_id_seq'::regclass);

ALTER TABLE ONLY trap_variables ALTER COLUMN id SET DEFAULT nextval('trap_variables_id_seq'::regclass);

SET search_path = netflow, pg_catalog;

ALTER TABLE ONLY alarm ALTER COLUMN id SET DEFAULT nextval('alarm_id_seq'::regclass);

ALTER TABLE ONLY counter_config ALTER COLUMN id SET DEFAULT nextval('counter_config_id_seq'::regclass);

ALTER TABLE ONLY default_config ALTER COLUMN id SET DEFAULT nextval('default_config_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_as_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_device_as_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_as_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_device_as_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_as_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_device_as_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_conversation_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_device_conversation_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_conversation_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_device_conversation_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_conversation_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_device_conversation_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_host_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_device_host_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_host_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_device_host_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_host_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_device_host_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_protocol_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_device_protocol_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_protocol_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_device_protocol_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_protocol_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_device_protocol_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_qos_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_device_qos_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_qos_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_device_qos_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_qos_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_device_qos_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_service_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_device_service_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_service_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_device_service_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_service_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_device_service_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_total_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_device_total_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_total_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_device_total_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_device_total_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_device_total_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_as_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_as_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_as_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_as_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_as_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_as_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_conversation_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_conversation_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_conversation_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_conversation_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_conversation_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_conversation_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_host_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_host_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_host_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_host_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_host_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_host_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_protocol_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_protocol_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_protocol_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_protocol_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_protocol_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_protocol_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_qos_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_qos_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_qos_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_qos_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_qos_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_qos_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_service_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_service_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_service_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_service_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_service_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_service_g3_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_total_g1 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_total_g1_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_total_g2 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_total_g2_id_seq'::regclass);

ALTER TABLE ONLY exporter_interface_total_g3 ALTER COLUMN id SET DEFAULT nextval('exporter_interface_total_g3_id_seq'::regclass);

ALTER TABLE ONLY pattern ALTER COLUMN id SET DEFAULT nextval('pattern_id_seq'::regclass);

ALTER TABLE ONLY pattern_criteria ALTER COLUMN id SET DEFAULT nextval('pattern_criteria_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_as_g1 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_as_g1_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_as_g2 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_as_g2_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_as_g3 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_as_g3_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_conversation_g1 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_conversation_g1_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_conversation_g2 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_conversation_g2_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_conversation_g3 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_conversation_g3_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_host_g1 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_host_g1_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_host_g2 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_host_g2_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_host_g3 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_host_g3_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_protocol_g1 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_protocol_g1_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_protocol_g2 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_protocol_g2_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_protocol_g3 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_protocol_g3_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_qos_g1 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_qos_g1_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_qos_g2 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_qos_g2_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_qos_g3 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_qos_g3_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_service_g1 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_service_g1_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_service_g2 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_service_g2_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_service_g3 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_service_g3_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_total_g1 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_total_g1_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_total_g2 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_total_g2_id_seq'::regclass);

ALTER TABLE ONLY pattern_subnet_total_g3 ALTER COLUMN id SET DEFAULT nextval('pattern_subnet_total_g3_id_seq'::regclass);

ALTER TABLE ONLY performance ALTER COLUMN id SET DEFAULT nextval('performance_id_seq'::regclass);

ALTER TABLE ONLY set_in_pattern ALTER COLUMN id SET DEFAULT nextval('set_in_pattern_id_seq'::regclass);

ALTER TABLE ONLY subnet ALTER COLUMN id SET DEFAULT nextval('subnet_id_seq'::regclass);

ALTER TABLE ONLY subnet_in_pattern ALTER COLUMN id SET DEFAULT nextval('subnet_in_pattern_id_seq'::regclass);

ALTER TABLE ONLY subnet_set ALTER COLUMN id SET DEFAULT nextval('subnet_set_id_seq'::regclass);

SET search_path = policy, pg_catalog;

ALTER TABLE ONLY policy_console ALTER COLUMN id_policy_console SET DEFAULT nextval('policy_console_id_policy_console_seq'::regclass);

ALTER TABLE ONLY policy_snmp ALTER COLUMN id_policy_snmp SET DEFAULT nextval('policy_snmp_id_policy_snmp_seq'::regclass);

SET search_path = report, pg_catalog;

ALTER TABLE ONLY report ALTER COLUMN id SET DEFAULT nextval('report_id_seq'::regclass);

ALTER TABLE ONLY report_query_from ALTER COLUMN id SET DEFAULT nextval('report_query_from_id_seq'::regclass);

ALTER TABLE ONLY report_query_join ALTER COLUMN id SET DEFAULT nextval('report_query_join_id_seq1'::regclass);

ALTER TABLE ONLY report_query_join_chunk ALTER COLUMN id SET DEFAULT nextval('report_query_join_id_seq'::regclass);

ALTER TABLE ONLY report_query_selects ALTER COLUMN id SET DEFAULT nextval('report_query_selects_id_seq'::regclass);

ALTER TABLE ONLY report_section ALTER COLUMN id SET DEFAULT nextval('report_section_id_seq'::regclass);

ALTER TABLE ONLY report_table_section_column ALTER COLUMN id SET DEFAULT nextval('report_table_section_column_id_seq'::regclass);

SET search_path = typology, pg_catalog;

ALTER TABLE ONLY device_type ALTER COLUMN id_device_type SET DEFAULT nextval('device_type_id_device_type_seq'::regclass);

ALTER TABLE ONLY resource_category ALTER COLUMN id_resource_category SET DEFAULT nextval('resource_category_id_resource_category_seq'::regclass);

ALTER TABLE ONLY resource_type ALTER COLUMN id_resource_type SET DEFAULT nextval('resource_type_id_resource_type_seq'::regclass);

ALTER TABLE ONLY subdevice_category ALTER COLUMN id_subdevice_category SET DEFAULT nextval('subdevice_category_id_subdevice_category_seq'::regclass);

ALTER TABLE ONLY subdevice_type ALTER COLUMN id_subdevice_type SET DEFAULT nextval('subdevice_type_id_subdevice_type_seq'::regclass);

ALTER TABLE ONLY value_type ALTER COLUMN id_value_type SET DEFAULT nextval('value_type_id_value_type_seq'::regclass);

SET search_path = view, pg_catalog;

ALTER TABLE ONLY bookmark_item ALTER COLUMN id_bookmark_item SET DEFAULT nextval('bookmark_item_id_bookmark_item_seq'::regclass);

SET search_path = core, pg_catalog;

ALTER TABLE ONLY criterion
    ADD CONSTRAINT criterion_pk PRIMARY KEY (id);

ALTER TABLE ONLY cip_sys_address_type
    ADD CONSTRAINT pk_cip_sys_address_type PRIMARY KEY (id_cip_sys_address_type);

ALTER TABLE ONLY cip_sys_module
    ADD CONSTRAINT pk_cip_sys_module PRIMARY KEY (id_cip_sys_module);

ALTER TABLE ONLY cip_usr_looking_glass_command
    ADD CONSTRAINT pk_cip_usr_looking_glass_command PRIMARY KEY (id_cip_usr_looking_glass_command);

ALTER TABLE ONLY favorites
    ADD CONSTRAINT pk_favorites PRIMARY KEY (id);

ALTER TABLE ONLY image
    ADD CONSTRAINT pk_image PRIMARY KEY (id_image);

ALTER TABLE ONLY permission_module
    ADD CONSTRAINT pk_permission_module PRIMARY KEY (id_permission_module);

ALTER TABLE ONLY property
    ADD CONSTRAINT pk_property PRIMARY KEY (id_property);

ALTER TABLE ONLY property_expression
    ADD CONSTRAINT pk_property_expression PRIMARY KEY (id_property);

ALTER TABLE ONLY property_regex
    ADD CONSTRAINT pk_property_regex PRIMARY KEY (id_property);

ALTER TABLE ONLY property_simple
    ADD CONSTRAINT pk_property_simple PRIMARY KEY (id_property);

ALTER TABLE ONLY property_snmp
    ADD CONSTRAINT pk_property_snmp PRIMARY KEY (id_property);

SET search_path = device, pg_catalog;

ALTER TABLE ONLY subdevice
    ADD CONSTRAINT dist_subdevice_on_device UNIQUE (id_device, id_subdevice_type, index);

ALTER TABLE ONLY device
    ADD CONSTRAINT pk_device PRIMARY KEY (id_device);

ALTER TABLE ONLY resource
    ADD CONSTRAINT pk_resource PRIMARY KEY (id_resource);

ALTER TABLE ONLY subdevice
    ADD CONSTRAINT pk_subdevice PRIMARY KEY (id_subdevice);

SET search_path = event_alarm_action, pg_catalog;

ALTER TABLE ONLY event_hidden
    ADD CONSTRAINT event_hidden_pk PRIMARY KEY (id_event_hidden);

ALTER TABLE ONLY event_resolution_type
    ADD CONSTRAINT event_resolution_type_pk PRIMARY KEY (id_resolution_type);

ALTER TABLE ONLY event_state
    ADD CONSTRAINT event_state_pk PRIMARY KEY (id_event_state);

ALTER TABLE ONLY event_ticket_state
    ADD CONSTRAINT event_ticket_state_pk PRIMARY KEY (id_ticket);

ALTER TABLE ONLY event
    ADD CONSTRAINT pk_event PRIMARY KEY (id_event);

SET search_path = filter, pg_catalog;

ALTER TABLE ONLY action
    ADD CONSTRAINT "PRIMARY_KEY" PRIMARY KEY (id);

ALTER TABLE ONLY action_set
    ADD CONSTRAINT "SERVICE_NAME" UNIQUE (service_name);

ALTER TABLE ONLY action_set
    ADD CONSTRAINT "action.set.primary" PRIMARY KEY (id);

SET search_path = flow, pg_catalog;

ALTER TABLE ONLY flow
    ADD CONSTRAINT flow_pk PRIMARY KEY (id);

ALTER TABLE ONLY media_application
    ADD CONSTRAINT media_application_pk PRIMARY KEY (id);

ALTER TABLE ONLY media_conversation
    ADD CONSTRAINT media_conv_pk PRIMARY KEY (id);

ALTER TABLE ONLY media_event
    ADD CONSTRAINT media_event_pk PRIMARY KEY (id);

ALTER TABLE ONLY media_sample
    ADD CONSTRAINT media_sample_pk PRIMARY KEY (id);

ALTER TABLE ONLY media_alarm
    ADD CONSTRAINT pk_media_alarm PRIMARY KEY (id);

ALTER TABLE ONLY media_alarm_stats
    ADD CONSTRAINT pk_media_alarm_stats PRIMARY KEY (id);

SET search_path = framework, pg_catalog;

ALTER TABLE ONLY module
    ADD CONSTRAINT ak_uk_order_module UNIQUE ("order");

ALTER TABLE ONLY module
    ADD CONSTRAINT ak_uk_tag_module UNIQUE (tag);

ALTER TABLE framework.db_version
	ADD CONSTRAINT db_version_pk PRIMARY KEY(id);

ALTER TABLE ONLY module
    ADD CONSTRAINT pk_module PRIMARY KEY (id_module);

SET search_path = login, pg_catalog;

ALTER TABLE ONLY principal
    ADD CONSTRAINT pk_role PRIMARY KEY (id_role);

ALTER TABLE ONLY "user"
    ADD CONSTRAINT pk_user PRIMARY KEY (id_role);

ALTER TABLE ONLY usergroup
    ADD CONSTRAINT pk_usergroup PRIMARY KEY (id_role);

ALTER TABLE ONLY usergroup_user
    ADD CONSTRAINT pk_usergroup_user PRIMARY KEY (id_usergroup_user);

ALTER TABLE ONLY "user"
    ADD CONSTRAINT uq_user_username UNIQUE (username);

SET search_path = mapping, pg_catalog;

ALTER TABLE ONLY asinfo
    ADD CONSTRAINT as_pkey PRIMARY KEY (asnumber);

ALTER TABLE ONLY service
    ADD CONSTRAINT pk_service PRIMARY KEY (id);

ALTER TABLE ONLY service_port
    ADD CONSTRAINT pk_service_port PRIMARY KEY (port);

ALTER TABLE ONLY protocol
    ADD CONSTRAINT protocol_pkey PRIMARY KEY (protocolnumber);

ALTER TABLE ONLY qos_dscp
    ADD CONSTRAINT qos_dscp_pkey PRIMARY KEY (dscp);

ALTER TABLE ONLY qos_ipprecedence
    ADD CONSTRAINT qos_ipprecedence_pkey PRIMARY KEY (ipprecedence);

ALTER TABLE ONLY qos_tos
    ADD CONSTRAINT qos_tos_pkey PRIMARY KEY (tosdecimal);

SET search_path = mib, pg_catalog;

ALTER TABLE ONLY element_module
    ADD CONSTRAINT element_id_pk PRIMARY KEY (element_id, module_id);

ALTER TABLE ONLY mib_favorite
    ADD CONSTRAINT mib_favorite_pk PRIMARY KEY (id);

ALTER TABLE ONLY mib_app_property
    ADD CONSTRAINT pk_mib_app_property PRIMARY KEY (id_mib_app_property);

ALTER TABLE ONLY mib_capability
    ADD CONSTRAINT pk_mib_capability PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_capability_support
    ADD CONSTRAINT pk_mib_capability_support PRIMARY KEY (id_capability_support);

ALTER TABLE ONLY mib_capability_support_variation
    ADD CONSTRAINT pk_mib_capability_support_variation PRIMARY KEY (id_capability_support_variation);

ALTER TABLE ONLY mib_compliance
    ADD CONSTRAINT pk_mib_compliance PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_compliance_module
    ADD CONSTRAINT pk_mib_compliance_module PRIMARY KEY (id_compliance_module);

ALTER TABLE ONLY mib_compliance_module_group
    ADD CONSTRAINT pk_mib_compliance_module_group PRIMARY KEY (id_compliance_module_group);

ALTER TABLE ONLY mib_compliance_module_object
    ADD CONSTRAINT pk_mib_compliance_module_object PRIMARY KEY (id_compliance_module_object);

ALTER TABLE ONLY mib_element
    ADD CONSTRAINT pk_mib_element PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_identity
    ADD CONSTRAINT pk_mib_identity PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_identity_revision
    ADD CONSTRAINT pk_mib_identity_revision PRIMARY KEY (id_identity_revision);

ALTER TABLE ONLY mib_module
    ADD CONSTRAINT pk_mib_module PRIMARY KEY (id_mib_module);

ALTER TABLE ONLY mib_module_import
    ADD CONSTRAINT pk_mib_module_import PRIMARY KEY (id_mib_module_import);

ALTER TABLE ONLY mib_notification_group
    ADD CONSTRAINT pk_mib_notification_group PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_notification_type
    ADD CONSTRAINT pk_mib_notification_type PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_object_group
    ADD CONSTRAINT pk_mib_object_group PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_object_type
    ADD CONSTRAINT pk_mib_object_type PRIMARY KEY (id_mib_element);

ALTER TABLE ONLY mib_textual_convention
    ADD CONSTRAINT pk_mib_textual_convention PRIMARY KEY (id_mib_textual_convention);

SET search_path = multilog, pg_catalog;

ALTER TABLE ONLY severity_exporter_counter
    ADD CONSTRAINT pk_exporter_severity_counter PRIMARY KEY (id);

ALTER TABLE ONLY performance
    ADD CONSTRAINT pk_performance PRIMARY KEY (id);

ALTER TABLE ONLY syslog_message
    ADD CONSTRAINT pk_syslog_message PRIMARY KEY (id);

ALTER TABLE ONLY trap_message
    ADD CONSTRAINT pk_trap_message PRIMARY KEY (id);

ALTER TABLE ONLY trap_variables
    ADD CONSTRAINT pk_trap_variables PRIMARY KEY (id);

SET search_path = netflow, pg_catalog;

ALTER TABLE ONLY counter_config
    ADD CONSTRAINT counter_conf_pk PRIMARY KEY (id);

ALTER TABLE ONLY default_config
    ADD CONSTRAINT default_config_pk PRIMARY KEY (id);

ALTER TABLE ONLY default_config
    ADD CONSTRAINT default_config_type_unique UNIQUE (type);

ALTER TABLE ONLY exporter_device_as_g1
    ADD CONSTRAINT exporter_device_as_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_as_g2
    ADD CONSTRAINT exporter_device_as_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_as_g3
    ADD CONSTRAINT exporter_device_as_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_conversation_g1
    ADD CONSTRAINT exporter_device_conversation_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_conversation_g2
    ADD CONSTRAINT exporter_device_conversation_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_conversation_g3
    ADD CONSTRAINT exporter_device_conversation_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_host_g1
    ADD CONSTRAINT exporter_device_host_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_host_g2
    ADD CONSTRAINT exporter_device_host_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_host_g3
    ADD CONSTRAINT exporter_device_host_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_protocol_g1
    ADD CONSTRAINT exporter_device_protocol_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_protocol_g2
    ADD CONSTRAINT exporter_device_protocol_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_protocol_g3
    ADD CONSTRAINT exporter_device_protocol_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_qos_g1
    ADD CONSTRAINT exporter_device_qos_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_qos_g2
    ADD CONSTRAINT exporter_device_qos_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_qos_g3
    ADD CONSTRAINT exporter_device_qos_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_service_g1
    ADD CONSTRAINT exporter_device_service_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_service_g2
    ADD CONSTRAINT exporter_device_service_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_service_g3
    ADD CONSTRAINT exporter_device_service_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_total_g1
    ADD CONSTRAINT exporter_device_total_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_total_g2
    ADD CONSTRAINT exporter_device_total_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_device_total_g3
    ADD CONSTRAINT exporter_device_total_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_as_g1
    ADD CONSTRAINT exporter_interface_as_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_as_g2
    ADD CONSTRAINT exporter_interface_as_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_as_g3
    ADD CONSTRAINT exporter_interface_as_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_conversation_g1
    ADD CONSTRAINT exporter_interface_conversation_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_conversation_g2
    ADD CONSTRAINT exporter_interface_conversation_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_conversation_g3
    ADD CONSTRAINT exporter_interface_conversation_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_host_g1
    ADD CONSTRAINT exporter_interface_host_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_host_g2
    ADD CONSTRAINT exporter_interface_host_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_host_g3
    ADD CONSTRAINT exporter_interface_host_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_protocol_g1
    ADD CONSTRAINT exporter_interface_protocol_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_protocol_g2
    ADD CONSTRAINT exporter_interface_protocol_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_protocol_g3
    ADD CONSTRAINT exporter_interface_protocol_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_qos_g1
    ADD CONSTRAINT exporter_interface_qos_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_qos_g2
    ADD CONSTRAINT exporter_interface_qos_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_qos_g3
    ADD CONSTRAINT exporter_interface_qos_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_service_g1
    ADD CONSTRAINT exporter_interface_service_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_service_g2
    ADD CONSTRAINT exporter_interface_service_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_service_g3
    ADD CONSTRAINT exporter_interface_service_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_total_g1
    ADD CONSTRAINT exporter_interface_total_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_total_g2
    ADD CONSTRAINT exporter_interface_total_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY exporter_interface_total_g3
    ADD CONSTRAINT exporter_interface_total_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_criteria
    ADD CONSTRAINT pattern_criteria_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern
    ADD CONSTRAINT pattern_pkey PRIMARY KEY (id);

ALTER TABLE ONLY set_in_pattern
    ADD CONSTRAINT pattern_set_unq UNIQUE (id_pattern, id_set);

ALTER TABLE ONLY pattern_subnet_as_g1
    ADD CONSTRAINT pattern_subnet_as_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_as_g2
    ADD CONSTRAINT pattern_subnet_as_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_as_g3
    ADD CONSTRAINT pattern_subnet_as_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_conversation_g1
    ADD CONSTRAINT pattern_subnet_conversation_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_conversation_g2
    ADD CONSTRAINT pattern_subnet_conversation_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_conversation_g3
    ADD CONSTRAINT pattern_subnet_conversation_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_host_g1
    ADD CONSTRAINT pattern_subnet_host_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_host_g2
    ADD CONSTRAINT pattern_subnet_host_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_host_g3
    ADD CONSTRAINT pattern_subnet_host_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_protocol_g1
    ADD CONSTRAINT pattern_subnet_protocol_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_protocol_g2
    ADD CONSTRAINT pattern_subnet_protocol_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_protocol_g3
    ADD CONSTRAINT pattern_subnet_protocol_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_qos_g1
    ADD CONSTRAINT pattern_subnet_qos_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_qos_g2
    ADD CONSTRAINT pattern_subnet_qos_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_qos_g3
    ADD CONSTRAINT pattern_subnet_qos_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_service_g1
    ADD CONSTRAINT pattern_subnet_service_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_service_g2
    ADD CONSTRAINT pattern_subnet_service_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_service_g3
    ADD CONSTRAINT pattern_subnet_service_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_total_g1
    ADD CONSTRAINT pattern_subnet_total_g1_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_total_g2
    ADD CONSTRAINT pattern_subnet_total_g2_pkey PRIMARY KEY (id);

ALTER TABLE ONLY pattern_subnet_total_g3
    ADD CONSTRAINT pattern_subnet_total_g3_pkey PRIMARY KEY (id);

ALTER TABLE ONLY subnet_in_pattern
    ADD CONSTRAINT pattern_subnet_unq UNIQUE (id_pattern, id_subnet);

ALTER TABLE ONLY performance
    ADD CONSTRAINT performance_pkey PRIMARY KEY (id);

ALTER TABLE ONLY alarm
    ADD CONSTRAINT pk_alarm PRIMARY KEY (id);

ALTER TABLE ONLY set_in_pattern
    ADD CONSTRAINT set_in_pattern_pk PRIMARY KEY (id);

ALTER TABLE ONLY subnet_set
    ADD CONSTRAINT set_pkey PRIMARY KEY (id);

ALTER TABLE ONLY subnet_in_pattern
    ADD CONSTRAINT subnet_in_pattern_pk PRIMARY KEY (id);

ALTER TABLE ONLY subnet
    ADD CONSTRAINT subnet_pkey PRIMARY KEY (id);

SET search_path = policy, pg_catalog;

ALTER TABLE ONLY device_policy_console
    ADD CONSTRAINT pk_device_policy_console PRIMARY KEY (id_device, id_policy_console);

ALTER TABLE ONLY device_policy_snmp
    ADD CONSTRAINT pk_device_policy_snmp PRIMARY KEY (id_device, id_policy_snmp);

ALTER TABLE ONLY policy_console
    ADD CONSTRAINT pk_policy_console PRIMARY KEY (id_policy_console);

ALTER TABLE ONLY policy_snmp
    ADD CONSTRAINT pk_policy_snmp PRIMARY KEY (id_policy_snmp);

SET search_path = public, pg_catalog;

ALTER TABLE ONLY system_property
    ADD CONSTRAINT pk_system_property PRIMARY KEY (name, tag);

ALTER TABLE ONLY user_property
    ADD CONSTRAINT pk_user_property PRIMARY KEY (name, tag, username);

SET search_path = report, pg_catalog;

ALTER TABLE ONLY report_table_section_column
    ADD CONSTRAINT pk_column PRIMARY KEY (id);

ALTER TABLE ONLY report_query_from
    ADD CONSTRAINT pk_from_query_element PRIMARY KEY (id);

ALTER TABLE ONLY report_query_selects
    ADD CONSTRAINT pk_id PRIMARY KEY (id);

ALTER TABLE ONLY report_query_join
    ADD CONSTRAINT pk_join PRIMARY KEY (id);

ALTER TABLE ONLY report
    ADD CONSTRAINT pk_report PRIMARY KEY (id);

ALTER TABLE ONLY report_section
    ADD CONSTRAINT pk_report_section PRIMARY KEY (id);

ALTER TABLE ONLY report_table_section
    ADD CONSTRAINT pk_report_table_section PRIMARY KEY (id);

ALTER TABLE ONLY report_query_join_chunk
    ADD CONSTRAINT report_query_join_pkey PRIMARY KEY (id);

SET search_path = typology, pg_catalog;

ALTER TABLE ONLY subdevice_type
    ADD CONSTRAINT dist_subdevice_type_on_device_type UNIQUE (id_device_type, id_subdevice_category);

ALTER TABLE ONLY resource_type
    ADD CONSTRAINT fk_resource_type PRIMARY KEY (id_resource_type);

ALTER TABLE ONLY subdevice_type
    ADD CONSTRAINT fk_subdevice_type PRIMARY KEY (id_subdevice_type);

ALTER TABLE ONLY device_type
    ADD CONSTRAINT pk_device_type PRIMARY KEY (id_device_type);

ALTER TABLE ONLY resource_category
    ADD CONSTRAINT pk_resorce_category PRIMARY KEY (id_resource_category);

ALTER TABLE ONLY subdevice_category
    ADD CONSTRAINT pk_subdevice_category PRIMARY KEY (id_subdevice_category);

ALTER TABLE ONLY value_type
    ADD CONSTRAINT pk_value_type PRIMARY KEY (id_value_type);

SET search_path = view, pg_catalog;

ALTER TABLE ONLY bookmark_item
    ADD CONSTRAINT pk_bookmark_item PRIMARY KEY (id_bookmark_item);

SET search_path = core, pg_catalog;

CREATE INDEX ix_permission_module_id_node_role ON permission_module USING btree (id_role);

SET search_path = event_alarm_action, pg_catalog;

CREATE UNIQUE INDEX event_ticket_state_idx ON event_ticket_state USING btree (id_event, action_time DESC NULLS LAST);

SET search_path = flow, pg_catalog;

CREATE INDEX conversation_dst_addr_idx ON media_conversation USING btree (dstaddr);

CREATE INDEX conversation_end_idx ON media_conversation USING btree (endtime);

CREATE INDEX conversation_src_addr_idx ON media_conversation USING btree (srcaddr);

CREATE INDEX conversation_start_idx ON media_conversation USING btree (starttime);

CREATE INDEX fki_media_event_event_fk ON media_event USING btree (id_event);

SET search_path = multilog, pg_catalog;

CREATE INDEX fki_trap_variable_trap_message ON trap_variables USING btree (owner_id);

CREATE INDEX idx_performance_time ON performance USING btree ("time");

CREATE INDEX idx_severity_exporter_counter_time ON severity_exporter_counter USING btree ("time");

CREATE INDEX idx_syslog_message_time ON syslog_message USING btree ("time");

SET search_path = netflow, pg_catalog;

CREATE INDEX exporter_device_as_g1_time_idx ON exporter_device_as_g1 USING btree ("time");

CREATE INDEX exporter_device_as_g2_time_idx ON exporter_device_as_g2 USING btree ("time");

CREATE INDEX exporter_device_as_g3_time_idx ON exporter_device_as_g3 USING btree ("time");

CREATE INDEX exporter_device_conversation_g1_time_idx ON exporter_device_conversation_g1 USING btree ("time");

CREATE INDEX exporter_device_conversation_g2_time_idx ON exporter_device_conversation_g2 USING btree ("time");

CREATE INDEX exporter_device_conversation_g3_time_idx ON exporter_device_conversation_g3 USING btree ("time");

CREATE INDEX exporter_device_host_g1_time_idx ON exporter_device_host_g1 USING btree ("time");

CREATE INDEX exporter_device_host_g2_time_idx ON exporter_device_host_g2 USING btree ("time");

CREATE INDEX exporter_device_host_g3_time_idx ON exporter_device_host_g3 USING btree ("time");

CREATE INDEX exporter_device_protocol_g1_time_idx ON exporter_device_protocol_g1 USING btree ("time");

CREATE INDEX exporter_device_protocol_g2_time_idx ON exporter_device_protocol_g2 USING btree ("time");

CREATE INDEX exporter_device_protocol_g3_time_idx ON exporter_device_protocol_g3 USING btree ("time");

CREATE INDEX exporter_device_qos_g1_time_idx ON exporter_device_qos_g1 USING btree ("time");

CREATE INDEX exporter_device_qos_g2_time_idx ON exporter_device_qos_g2 USING btree ("time");

CREATE INDEX exporter_device_qos_g3_time_idx ON exporter_device_qos_g3 USING btree ("time");

CREATE INDEX exporter_device_service_g1_time_idx ON exporter_device_service_g1 USING btree ("time");

CREATE INDEX exporter_device_service_g2_time_idx ON exporter_device_service_g2 USING btree ("time");

CREATE INDEX exporter_device_service_g3_time_idx ON exporter_device_service_g3 USING btree ("time");

CREATE INDEX exporter_device_total_g1_time_idx ON exporter_device_total_g1 USING btree ("time");

CREATE INDEX exporter_device_total_g2_time_idx ON exporter_device_total_g2 USING btree ("time");

CREATE INDEX exporter_device_total_g3_time_idx ON exporter_device_total_g3 USING btree ("time");

CREATE INDEX exporter_interface_as_g1_time_idx ON exporter_interface_as_g1 USING btree ("time");

CREATE INDEX exporter_interface_as_g2_time_idx ON exporter_interface_as_g2 USING btree ("time");

CREATE INDEX exporter_interface_as_g3_time_idx ON exporter_interface_as_g3 USING btree ("time");

CREATE INDEX exporter_interface_conversation_g1_time_idx ON exporter_interface_conversation_g1 USING btree ("time");

CREATE INDEX exporter_interface_conversation_g2_time_idx ON exporter_interface_conversation_g2 USING btree ("time");

CREATE INDEX exporter_interface_conversation_g3_time_idx ON exporter_interface_conversation_g3 USING btree ("time");

CREATE INDEX exporter_interface_host_g1_time_idx ON exporter_interface_host_g1 USING btree ("time");

CREATE INDEX exporter_interface_host_g2_time_idx ON exporter_interface_host_g2 USING btree ("time");

CREATE INDEX exporter_interface_host_g3_time_idx ON exporter_interface_host_g3 USING btree ("time");

CREATE INDEX exporter_interface_protocol_g1_time_idx ON exporter_interface_protocol_g1 USING btree ("time");

CREATE INDEX exporter_interface_protocol_g2_time_idx ON exporter_interface_protocol_g2 USING btree ("time");

CREATE INDEX exporter_interface_protocol_g3_time_idx ON exporter_interface_protocol_g3 USING btree ("time");

CREATE INDEX exporter_interface_qos_g1_time_idx ON exporter_interface_qos_g1 USING btree ("time");

CREATE INDEX exporter_interface_qos_g2_time_idx ON exporter_interface_qos_g2 USING btree ("time");

CREATE INDEX exporter_interface_qos_g3_time_idx ON exporter_interface_qos_g3 USING btree ("time");

CREATE INDEX exporter_interface_service_g1_time_idx ON exporter_interface_service_g1 USING btree ("time");

CREATE INDEX exporter_interface_service_g2_time_idx ON exporter_interface_service_g2 USING btree ("time");

CREATE INDEX exporter_interface_service_g3_time_idx ON exporter_interface_service_g3 USING btree ("time");

CREATE INDEX exporter_interface_total_g1_time_idx ON exporter_interface_total_g1 USING btree ("time");

CREATE INDEX exporter_interface_total_g2_time_idx ON exporter_interface_total_g2 USING btree ("time");

CREATE INDEX exporter_interface_total_g3_time_idx ON exporter_interface_total_g3 USING btree ("time");

CREATE INDEX fki_set_set ON subnet_set USING btree (id_parent);

CREATE INDEX fki_subnet_set ON subnet USING btree (id_set);

CREATE INDEX fki_subnet_subnet ON subnet USING btree (id_parent);

CREATE INDEX pattern_subnet_as_g1_time_idx ON pattern_subnet_as_g1 USING btree ("time");

CREATE INDEX pattern_subnet_as_g2_time_idx ON pattern_subnet_as_g2 USING btree ("time");

CREATE INDEX pattern_subnet_as_g3_time_idx ON pattern_subnet_as_g3 USING btree ("time");

CREATE INDEX pattern_subnet_conversation_g1_time_idx ON pattern_subnet_conversation_g1 USING btree ("time");

CREATE INDEX pattern_subnet_conversation_g2_time_idx ON pattern_subnet_conversation_g2 USING btree ("time");

CREATE INDEX pattern_subnet_conversation_g3_time_idx ON pattern_subnet_conversation_g3 USING btree ("time");

CREATE INDEX pattern_subnet_host_g1_time_idx ON pattern_subnet_host_g1 USING btree ("time");

CREATE INDEX pattern_subnet_host_g2_time_idx ON pattern_subnet_host_g2 USING btree ("time");

CREATE INDEX pattern_subnet_host_g3_time_idx ON pattern_subnet_host_g3 USING btree ("time");

CREATE INDEX pattern_subnet_protocol_g1_time_idx ON pattern_subnet_protocol_g1 USING btree ("time");

CREATE INDEX pattern_subnet_protocol_g2_time_idx ON pattern_subnet_protocol_g2 USING btree ("time");

CREATE INDEX pattern_subnet_protocol_g3_time_idx ON pattern_subnet_protocol_g3 USING btree ("time");

CREATE INDEX pattern_subnet_qos_g1_time_idx ON pattern_subnet_qos_g1 USING btree ("time");

CREATE INDEX pattern_subnet_qos_g2_time_idx ON pattern_subnet_qos_g2 USING btree ("time");

CREATE INDEX pattern_subnet_qos_g3_time_idx ON pattern_subnet_qos_g3 USING btree ("time");

CREATE INDEX pattern_subnet_service_g1_time_idx ON pattern_subnet_service_g1 USING btree ("time");

CREATE INDEX pattern_subnet_service_g2_time_idx ON pattern_subnet_service_g2 USING btree ("time");

CREATE INDEX pattern_subnet_service_g3_time_idx ON pattern_subnet_service_g3 USING btree ("time");

CREATE INDEX pattern_subnet_total_g1_time_idx ON pattern_subnet_total_g1 USING btree ("time");

CREATE INDEX pattern_subnet_total_g2_time_idx ON pattern_subnet_total_g2 USING btree ("time");

CREATE INDEX pattern_subnet_total_g3_time_idx ON pattern_subnet_total_g3 USING btree ("time");

CREATE INDEX performance_time_idx ON performance USING btree ("time");

SET search_path = core, pg_catalog;

ALTER TABLE ONLY criterion
    ADD CONSTRAINT criterion_criterion FOREIGN KEY (owner_id) REFERENCES criterion(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY permission_module
    ADD CONSTRAINT fk_permission_module_cip_sys_module FOREIGN KEY (id_cip_sys_module) REFERENCES cip_sys_module(id_cip_sys_module);

ALTER TABLE ONLY permission_module
    ADD CONSTRAINT fk_permission_module_user FOREIGN KEY (id_role) REFERENCES login."user"(id_role) ON DELETE CASCADE;

ALTER TABLE ONLY property_expression
    ADD CONSTRAINT fk_property_expression_property FOREIGN KEY (id_property) REFERENCES property(id_property) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY property_regex
    ADD CONSTRAINT fk_property_regex_property FOREIGN KEY (id_property) REFERENCES property(id_property) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY property_simple
    ADD CONSTRAINT fk_property_simple_property FOREIGN KEY (id_property) REFERENCES property(id_property) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY property_snmp
    ADD CONSTRAINT fk_property_snmp_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib.mib_element(id_mib_element);

ALTER TABLE ONLY property_snmp
    ADD CONSTRAINT fk_property_snmp_property FOREIGN KEY (id_property) REFERENCES property(id_property) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY favorites
    ADD CONSTRAINT fk_user_favorites FOREIGN KEY (id_user) REFERENCES login."user"(id_role) ON DELETE CASCADE;

SET search_path = device, pg_catalog;

ALTER TABLE ONLY resource
    ADD CONSTRAINT fk_resource_device FOREIGN KEY (id_device) REFERENCES device(id_device) MATCH FULL ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY resource
    ADD CONSTRAINT fk_resource_resource_type FOREIGN KEY (id_resource_type) REFERENCES typology.resource_type(id_resource_type) MATCH FULL;

ALTER TABLE ONLY resource
    ADD CONSTRAINT fk_resource_subdevice FOREIGN KEY (id_subdevice) REFERENCES subdevice(id_subdevice) MATCH FULL ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY subdevice
    ADD CONSTRAINT fk_subdevice_device FOREIGN KEY (id_device) REFERENCES device(id_device) MATCH FULL ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY subdevice
    ADD CONSTRAINT fk_subdevice_parent FOREIGN KEY (id_parent) REFERENCES subdevice(id_subdevice) MATCH FULL ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY subdevice
    ADD CONSTRAINT fk_subdevice_subdevice_type FOREIGN KEY (id_subdevice_type) REFERENCES typology.subdevice_type(id_subdevice_type);

SET search_path = event_alarm_action, pg_catalog;

ALTER TABLE ONLY event_state
    ADD CONSTRAINT action_by_user_fk FOREIGN KEY (action_user) REFERENCES login."user"(id_role);

ALTER TABLE ONLY event_ticket_state
    ADD CONSTRAINT action_user_event_ticket_state_fk FOREIGN KEY (action_by_user) REFERENCES login."user"(id_role);

ALTER TABLE ONLY event_ticket_state
    ADD CONSTRAINT assigned_user_event_ticket_state FOREIGN KEY (assigned_to_user) REFERENCES login."user"(id_role);

ALTER TABLE ONLY event_state
    ADD CONSTRAINT event_event_state FOREIGN KEY (id_event) REFERENCES event(id_event) ON DELETE CASCADE;

ALTER TABLE ONLY event_ticket_state
    ADD CONSTRAINT event_event_ticket_state_fk FOREIGN KEY (id_event) REFERENCES event(id_event) ON DELETE CASCADE;

ALTER TABLE ONLY event_hidden
    ADD CONSTRAINT event_hidden_event FOREIGN KEY (id_event) REFERENCES event(id_event) ON DELETE CASCADE;

ALTER TABLE ONLY event_hidden
    ADD CONSTRAINT event_hidden_user_fk FOREIGN KEY (id_user) REFERENCES login."user"(id_role) ON DELETE CASCADE;

ALTER TABLE ONLY event_state
    ADD CONSTRAINT event_state_resolution_type_fk FOREIGN KEY (resolution_type) REFERENCES event_resolution_type(id_resolution_type);

ALTER TABLE ONLY event
    ADD CONSTRAINT fk_event_module_origin FOREIGN KEY (id_cip_sys_module) REFERENCES core.cip_sys_module(id_cip_sys_module);

SET search_path = filter, pg_catalog;

ALTER TABLE ONLY action
    ADD CONSTRAINT "CRITERION_ID" FOREIGN KEY (criterion_id) REFERENCES core.criterion(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY action
    ADD CONSTRAINT "OWNER_ID" FOREIGN KEY (owner_id) REFERENCES action_set(id) ON UPDATE CASCADE ON DELETE CASCADE;

SET search_path = flow, pg_catalog;

ALTER TABLE ONLY media_alarm
    ADD CONSTRAINT fk_media_alarm_criterion FOREIGN KEY (criterion) REFERENCES core.criterion(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY media_application
    ADD CONSTRAINT media_application_fk_criterion FOREIGN KEY (criterion) REFERENCES core.criterion(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY media_event
    ADD CONSTRAINT media_event_conversation_fk FOREIGN KEY (id_conversation) REFERENCES media_conversation(id);

ALTER TABLE ONLY media_event
    ADD CONSTRAINT media_event_event_fk FOREIGN KEY (id_event) REFERENCES event_alarm_action.event(id_event) ON DELETE CASCADE;

ALTER TABLE ONLY media_sample
    ADD CONSTRAINT media_sample_conversation_fk FOREIGN KEY (id_conversation) REFERENCES media_conversation(id);

ALTER TABLE ONLY media_sample
    ADD CONSTRAINT media_sample_flow_fk FOREIGN KEY (id_flow) REFERENCES flow(id) ON DELETE CASCADE;

SET search_path = login, pg_catalog;

ALTER TABLE ONLY principal
    ADD CONSTRAINT fk_role_role_c FOREIGN KEY (id_role_c) REFERENCES principal(id_role);

ALTER TABLE ONLY principal
    ADD CONSTRAINT fk_role_role_m FOREIGN KEY (id_role_m) REFERENCES principal(id_role);

ALTER TABLE ONLY "user"
    ADD CONSTRAINT fk_user_role FOREIGN KEY (id_role) REFERENCES principal(id_role);

ALTER TABLE ONLY usergroup
    ADD CONSTRAINT fk_usergroup_role FOREIGN KEY (id_role) REFERENCES principal(id_role);

ALTER TABLE ONLY usergroup_user
    ADD CONSTRAINT fk_usergroup_user_c_user FOREIGN KEY (id_role_c) REFERENCES "user"(id_role);

ALTER TABLE ONLY usergroup_user
    ADD CONSTRAINT fk_usergroup_user_d_user FOREIGN KEY (id_role_d) REFERENCES "user"(id_role);

ALTER TABLE ONLY usergroup_user
    ADD CONSTRAINT fk_usergroup_user_user FOREIGN KEY (id_role_user) REFERENCES "user"(id_role);

ALTER TABLE ONLY usergroup_user
    ADD CONSTRAINT fk_usergroup_user_usergroup FOREIGN KEY (id_role_usergroup) REFERENCES usergroup(id_role);

SET search_path = mapping, pg_catalog;

ALTER TABLE ONLY service_port
    ADD CONSTRAINT fk_service FOREIGN KEY (id_service) REFERENCES service(id) ON UPDATE CASCADE ON DELETE CASCADE;

SET search_path = mib, pg_catalog;

ALTER TABLE ONLY element_module
    ADD CONSTRAINT element_module_element_fk FOREIGN KEY (element_id) REFERENCES mib_element(id_mib_element);

ALTER TABLE ONLY element_module
    ADD CONSTRAINT element_module_module_fk FOREIGN KEY (module_id) REFERENCES mib_module(id_mib_module);

ALTER TABLE ONLY mib_capability
    ADD CONSTRAINT fk_mib_capability_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_capability_support
    ADD CONSTRAINT fk_mib_capability_support_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_capability_support_variation
    ADD CONSTRAINT fk_mib_capability_support_variation_mib_capability_support FOREIGN KEY (id_capability_support) REFERENCES mib_capability_support(id_capability_support) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_compliance
    ADD CONSTRAINT fk_mib_compliance_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_compliance_module_group
    ADD CONSTRAINT fk_mib_compliance_module_group_mib_compliance_module FOREIGN KEY (id_compliance_module) REFERENCES mib_compliance_module(id_compliance_module) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_compliance_module
    ADD CONSTRAINT fk_mib_compliance_module_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_compliance_module_object
    ADD CONSTRAINT fk_mib_compliance_module_object_mib_compliance_module FOREIGN KEY (id_compliance_module) REFERENCES mib_compliance_module(id_compliance_module) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_element
    ADD CONSTRAINT fk_mib_element_mib_element FOREIGN KEY (id_mib_element_parent) REFERENCES mib_element(id_mib_element);

ALTER TABLE ONLY mib_identity
    ADD CONSTRAINT fk_mib_identity_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_identity_revision
    ADD CONSTRAINT fk_mib_identity_revision_mib_identity FOREIGN KEY (id_mib_element) REFERENCES mib_identity(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_module_import
    ADD CONSTRAINT fk_mib_module_import_mib_module FOREIGN KEY (id_mib_module) REFERENCES mib_module(id_mib_module) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_module_import
    ADD CONSTRAINT fk_mib_module_import_mib_module_from FOREIGN KEY (id_mib_module_import_from) REFERENCES mib_module(id_mib_module);

ALTER TABLE ONLY mib_notification_group
    ADD CONSTRAINT fk_mib_notification_group_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_notification_type
    ADD CONSTRAINT fk_mib_notification_type_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_object_group
    ADD CONSTRAINT fk_mib_object_group_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_object_type
    ADD CONSTRAINT fk_mib_object_type_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_textual_convention
    ADD CONSTRAINT fk_mib_textual_convention_mib_module FOREIGN KEY (id_mib_module) REFERENCES mib_module(id_mib_module) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY mib_favorite
    ADD CONSTRAINT mib_favorite_fk_mib_element FOREIGN KEY (id_element) REFERENCES mib_element(id_mib_element);

ALTER TABLE ONLY mib_favorite
    ADD CONSTRAINT mib_favorite_fk_user FOREIGN KEY (id_user) REFERENCES login."user"(id_role);

SET search_path = multilog, pg_catalog;

ALTER TABLE ONLY trap_variables
    ADD CONSTRAINT fk_trap_variable_trap_message FOREIGN KEY (owner_id) REFERENCES trap_message(id) ON UPDATE CASCADE ON DELETE CASCADE;

SET search_path = netflow, pg_catalog;

ALTER TABLE ONLY counter_config
    ADD CONSTRAINT counter_conf_fk_pattern FOREIGN KEY (pattern) REFERENCES pattern(id) ON DELETE CASCADE;

ALTER TABLE ONLY counter_config
    ADD CONSTRAINT counter_conf_fk_subnet FOREIGN KEY (subnet) REFERENCES subnet(id) ON DELETE CASCADE;

ALTER TABLE ONLY counter_config
    ADD CONSTRAINT counter_conf_fk_subnet_set FOREIGN KEY (subnet_set) REFERENCES subnet_set(id);

ALTER TABLE ONLY alarm
    ADD CONSTRAINT fk_alarm_criterion FOREIGN KEY (criterion) REFERENCES core.criterion(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY subnet_set
    ADD CONSTRAINT fk_set_set FOREIGN KEY (id_parent) REFERENCES subnet_set(id);

ALTER TABLE ONLY subnet
    ADD CONSTRAINT fk_subnet_set FOREIGN KEY (id_set) REFERENCES subnet_set(id);

ALTER TABLE ONLY subnet
    ADD CONSTRAINT fk_subnet_subnet FOREIGN KEY (id_parent) REFERENCES subnet(id);

ALTER TABLE ONLY pattern_criteria
    ADD CONSTRAINT pattern_criteria_pattern_fkey FOREIGN KEY (pattern_id) REFERENCES pattern(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY subnet_in_pattern
    ADD CONSTRAINT pattern_fk FOREIGN KEY (id_pattern) REFERENCES pattern(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY set_in_pattern
    ADD CONSTRAINT pattern_set_fk FOREIGN KEY (id_pattern) REFERENCES pattern(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY set_in_pattern
    ADD CONSTRAINT set_pattern_fk FOREIGN KEY (id_set) REFERENCES subnet_set(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY subnet_in_pattern
    ADD CONSTRAINT subnet_fk FOREIGN KEY (id_subnet) REFERENCES subnet(id) ON UPDATE CASCADE ON DELETE CASCADE;

SET search_path = policy, pg_catalog;

ALTER TABLE ONLY device_policy_console
    ADD CONSTRAINT fk_device_policy_console_device FOREIGN KEY (id_device) REFERENCES device.device(id_device) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY device_policy_console
    ADD CONSTRAINT fk_device_policy_console_policy_console FOREIGN KEY (id_policy_console) REFERENCES policy_console(id_policy_console) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY device_policy_snmp
    ADD CONSTRAINT fk_device_policy_snmp_device FOREIGN KEY (id_device) REFERENCES device.device(id_device) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY device_policy_snmp
    ADD CONSTRAINT fk_device_policy_snmp_policy_snmp FOREIGN KEY (id_policy_snmp) REFERENCES policy_snmp(id_policy_snmp) ON UPDATE CASCADE ON DELETE CASCADE;

SET search_path = report, pg_catalog;

ALTER TABLE ONLY report_query_from
    ADD CONSTRAINT fk_from_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib.mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY report_query_join_chunk
    ADD CONSTRAINT fk_join FOREIGN KEY (id_join) REFERENCES report_query_join(id);

ALTER TABLE ONLY report_query_join_chunk
    ADD CONSTRAINT fk_mib FOREIGN KEY (id_mib_element) REFERENCES mib.mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY report_table_section_column
    ADD CONSTRAINT fk_mib FOREIGN KEY (id_mib_element) REFERENCES mib.mib_element(id_mib_element);

ALTER TABLE ONLY report_query_selects
    ADD CONSTRAINT fk_mib_element FOREIGN KEY (id_mib_element) REFERENCES mib.mib_element(id_mib_element) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY report_query_selects
    ADD CONSTRAINT fk_mib_table FOREIGN KEY (id_mib_table) REFERENCES mib.mib_element(id_mib_element);

ALTER TABLE ONLY report_query_selects
    ADD CONSTRAINT fk_parent_select FOREIGN KEY (id_parent) REFERENCES report_query_selects(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY report_table_section_column
    ADD CONSTRAINT fk_query_element FOREIGN KEY (id_report_query_select) REFERENCES report_query_selects(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY report_section
    ADD CONSTRAINT fk_report_section_report FOREIGN KEY (id_report) REFERENCES report(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY report_table_section
    ADD CONSTRAINT fk_report_table_section_report_section FOREIGN KEY (id) REFERENCES report_section(id) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY report_query_join
    ADD CONSTRAINT fk_section FOREIGN KEY (id_section) REFERENCES report_section(id);

ALTER TABLE ONLY report_query_from
    ADD CONSTRAINT fk_section FOREIGN KEY (id_section) REFERENCES report_section(id);

ALTER TABLE ONLY report_table_section_column
    ADD CONSTRAINT fk_section FOREIGN KEY (id_report_table_section) REFERENCES report_table_section(id) ON UPDATE CASCADE ON DELETE CASCADE;

SET search_path = typology, pg_catalog;

ALTER TABLE ONLY device_device_type
    ADD CONSTRAINT fk_device_device_type FOREIGN KEY (id_device) REFERENCES device.device(id_device) MATCH FULL;

ALTER TABLE ONLY device_device_type
    ADD CONSTRAINT fk_device_device_type_device_type FOREIGN KEY (id_device_type) REFERENCES device_type(id_device_type) MATCH FULL;

ALTER TABLE ONLY device_type
    ADD CONSTRAINT fk_device_type_criterion FOREIGN KEY (id_test) REFERENCES core.criterion(id);

ALTER TABLE ONLY device_type
    ADD CONSTRAINT fk_device_type_parent FOREIGN KEY (id_parent) REFERENCES device_type(id_device_type) MATCH FULL ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY resource_category
    ADD CONSTRAINT fk_resource_category_subdevice_category FOREIGN KEY (id_subdevice_category) REFERENCES subdevice_category(id_subdevice_category) MATCH FULL;

ALTER TABLE ONLY resource_type
    ADD CONSTRAINT fk_resource_type_device_type FOREIGN KEY (id_device_type) REFERENCES device_type(id_device_type) MATCH FULL;

ALTER TABLE ONLY resource_type
    ADD CONSTRAINT fk_resource_type_resource_category FOREIGN KEY (id_resource_category) REFERENCES resource_category(id_resource_category) MATCH FULL;

ALTER TABLE ONLY resource_type
    ADD CONSTRAINT fk_resource_type_subdevice_type FOREIGN KEY (id_subdevice_type) REFERENCES subdevice_type(id_subdevice_type) MATCH FULL;

ALTER TABLE ONLY resource_type
    ADD CONSTRAINT fk_resource_type_value_type FOREIGN KEY (id_value_type) REFERENCES value_type(id_value_type) MATCH FULL;

ALTER TABLE ONLY subdevice_type
    ADD CONSTRAINT fk_subdevice_type_device_type FOREIGN KEY (id_device_type) REFERENCES device_type(id_device_type) MATCH FULL;

ALTER TABLE ONLY subdevice_type
    ADD CONSTRAINT fk_subdevice_type_parent FOREIGN KEY (id_parent) REFERENCES subdevice_type(id_subdevice_type) MATCH FULL ON UPDATE RESTRICT ON DELETE RESTRICT;

ALTER TABLE ONLY subdevice_type
    ADD CONSTRAINT fk_subdevice_type_subdevice_category FOREIGN KEY (id_subdevice_category) REFERENCES subdevice_category(id_subdevice_category) MATCH FULL;

SET search_path = view, pg_catalog;

ALTER TABLE ONLY bookmark_item
    ADD CONSTRAINT fk_bookmark_item_user FOREIGN KEY (id_role) REFERENCES login."user"(id_role) ON UPDATE CASCADE ON DELETE CASCADE;

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;
